import 'dart:io';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_background/flutter_background.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'app_opener.dart';

///
///设备环境类型
///
enum AppConfigDeviceType {
  ///web浏览器
  web,

  ///andrid移动设备
  android,

  ///ios移动设备
  ios,

  ///mac桌面设备
  macOS,

  ///linux桌面设备
  linux,

  ///windows桌面设备
  windows,

  ///fuchsia桌面设备
  fuchsia,

  ///others桌面设备
  others,
}

///
///配置信息类
///
class AppConfig {
  /* ******************************** public static properties ******************************** */

  ///全局可滑动部件滚动物理效果
  static const globalScrollPhysics = BouncingScrollPhysics();

  ///图标的尺寸-超小
  static const iconGlobalSizeMini = 12.0;

  ///图标的尺寸-小
  static const iconGlobalSizeSmall = 16.0;

  ///图标的尺寸-中
  static const iconGlobalSizeMedium = 20.0;

  ///图标的尺寸-大
  static const iconGlobalSizeLarge = 24.0;

  ///主内容内边距-小
  static const bodyGlobalPaddingMini = EdgeInsets.all(8);

  ///主内容内边距-小
  static const bodyGlobalPaddingSmall = EdgeInsets.all(12);

  ///主内容内边距-中
  static const bodyGlobalPaddingMedium = EdgeInsets.all(16);

  ///主内容内边距-大
  static const bodyGlobalPaddingLarge = EdgeInsets.all(20);

  ///列表可滑动项动画
  static const listGlobalItemMotion = DrawerMotion();

  ///列表分页加载动画
  static const listGlobalPageLoading = SizedBox(width: 20, height: 20, child: CircularProgressIndicator());

  ///卡片间隔-小
  static const cardGlobalSpaceSmall = SizedBox(height: 8);

  ///卡片间隔-中
  static const cardGlobalSpaceMedium = SizedBox(height: 16);

  ///卡片间隔-大
  static const cardGlobalSpaceLarge = SizedBox(height: 24);

  ///信息页列表头内边距
  static const infoListTileHeadPadding = EdgeInsets.only(left: 16, right: 12, top: 4, bottom: 4);

  ///信息页列表项内边距
  static const infoListTileItemPadding = EdgeInsets.only(left: 16, right: 12, top: 0, bottom: 0);

  ///信息页列表项尾部箭头图标
  static Icon get infoListTileTailArrowIC => Icon(Icons.navigate_next_outlined, size: iconGlobalSizeMedium, color: themeData.textTheme.caption?.color);

  /* ******************************** private static properties ******************************** */

  static late final AppConfigDeviceType _deviceType;

  ///日志处理器
  static late final EasyLogger _easyLogger;

  ///持久化实例
  static late final SharedPreferences? _sharedPrefs;

  ///安装包信息
  static late final PackageInfo _packageInfo;

  ///设备信息
  static late final Map<String, dynamic> _deviceInfo;

  ///浏览器agent
  static late final String _agentInfo;

  ///网络客户端
  static late final NetClient _netClient;

  ///安卓保活运行（即是否启用 https://pub.dev/packages/flutter_background 插件）
  static late final bool _androidKeepAlive;

  ///本地通知插件
  static late final FlutterLocalNotificationsPlugin? _notifications;

  ///夜间模式主题
  static late final ThemeData _darkThemeData;

  ///日间模式主题
  static late final ThemeData _lightThemeData;

  ///男性图标
  static late final Icon _sexIconMale;

  ///女性图标
  static late final Icon _sexIconFemale;

  ///是否已经切换到后台
  static bool _runBackground = false;

  ///是否同意协议和政策
  static bool _agreeProtocol = false;

  ///当前是否为夜间模式
  static bool _useDarkTheme = false;

  ///自定义虚拟键盘高度
  static double _keyboardHeight = 250.0;

  ///自定义虚拟键盘高度
  static int _notificationIdInc = 0;

  ///协议和政策同意状态变化监听函数
  static final List<void Function()> _onAgreeProtocolChange = [];

  ///主题日间模式、夜间模式变化监听函数
  static final List<void Function()> _onBrightnessChange = [];

  /* ******************************** public static functions ******************************** */
  ///初始化配置信息
  static void init({
    //webview的Agent标志
    required String agentFlag,
    //网络库配置
    required EasyLogLevel logLevel,
    required String host,
    required String bsid,
    required String secret,
    //安卓保活配置
    bool androidKeepAliveEnable = false,
    FlutterBackgroundAndroidConfig? androidKeepAliveConfig,
    //本地通知配置
    bool notificationsEnable = false,
    AndroidInitializationSettings? notificationAndroidSettings,
    IOSInitializationSettings? notificationIosSettings,
    MacOSInitializationSettings? notificationMacOSSettings,
    LinuxInitializationSettings? notificationLinuxSettings,
    //开放平台配置
    String? amapAndroidKey,
    String? amapIosKey,
    String? amapWebKey,
    String? wechatAppId,
    String? wechatUniversalLink,
    //主题配置
    MaterialColor themeColor = Colors.teal,
    MaterialAccentColor themeAccentColor = Colors.tealAccent,
    ThemeData? darkThemeData,
    ThemeData? lightThemeData,
    Icon? sexIconMale,
    Icon? sexIconFemale,
    //回调函数
    required void Function(ObjectId? uid, String? token) onInited,
  }) async {
    assert((amapAndroidKey == null && amapIosKey == null && amapWebKey == null) || (amapAndroidKey != null && amapIosKey != null && amapWebKey != null));
    assert((wechatAppId == null && wechatUniversalLink == null) || (wechatAppId != null && wechatUniversalLink != null));
    WidgetsFlutterBinding.ensureInitialized();
    //设备类型
    if (kIsWeb) {
      _deviceType = AppConfigDeviceType.web;
    } else if (Platform.isAndroid) {
      _deviceType = AppConfigDeviceType.android;
    } else if (Platform.isIOS) {
      _deviceType = AppConfigDeviceType.ios;
    } else if (Platform.isMacOS) {
      _deviceType = AppConfigDeviceType.macOS;
    } else if (Platform.isLinux) {
      _deviceType = AppConfigDeviceType.linux;
    } else if (Platform.isWindows) {
      _deviceType = AppConfigDeviceType.windows;
    } else if (Platform.isFuchsia) {
      _deviceType = AppConfigDeviceType.fuchsia;
    } else {
      _deviceType = AppConfigDeviceType.others;
    }
    //_easyLogger
    _easyLogger = EasyLogger(
      logger: _deviceType == AppConfigDeviceType.ios ? EasyLogger.stdoutLogger : EasyLogger.printLogger,
      logLevel: logLevel,
      logTag: '[${_deviceType.name}]',
    );
    //_sharedPrefs
    try {
      _sharedPrefs = await SharedPreferences.getInstance();
    } catch (e) {
      _sharedPrefs = null;
    }
    //_packageInfo
    try {
      _packageInfo = await PackageInfo.fromPlatform();
    } catch (e) {
      _packageInfo = PackageInfo(
        appName: _deviceType.name,
        packageName: _deviceType.name,
        version: _deviceType.name,
        buildNumber: _deviceType.name,
      );
    }
    //_deviceInfo and _agentInfo
    try {
      switch (_deviceType) {
        case AppConfigDeviceType.web:
          final webBrowserInfo = await DeviceInfoPlugin().webBrowserInfo;
          _deviceInfo = webBrowserInfo.toMap();
          _agentInfo = 'Browser ${webBrowserInfo.appVersion}; ${webBrowserInfo.browserName.name}; ${agentFlag}Messenger ${_packageInfo.version}';
          break;
        case AppConfigDeviceType.android:
          final androidInfo = await DeviceInfoPlugin().androidInfo;
          _deviceInfo = androidInfo.toMap();
          _agentInfo = 'Android ${androidInfo.version.release}; ${androidInfo.model}; ${agentFlag}Messenger ${_packageInfo.version}';
          break;
        case AppConfigDeviceType.ios:
          final iosInfo = await DeviceInfoPlugin().iosInfo;
          _deviceInfo = iosInfo.toMap();
          _agentInfo = 'IOS ${iosInfo.systemVersion}; ${iosInfo.model}; ${agentFlag}Messenger ${_packageInfo.version}';
          break;
        case AppConfigDeviceType.macOS:
          final macOsInfo = await DeviceInfoPlugin().macOsInfo;
          _deviceInfo = macOsInfo.toMap();
          _agentInfo = 'MacOS ${macOsInfo.osRelease}; ${macOsInfo.model}; ${agentFlag}Messenger ${_packageInfo.version}';
          break;
        case AppConfigDeviceType.linux:
          final linuxInfo = await DeviceInfoPlugin().linuxInfo;
          _deviceInfo = linuxInfo.toMap();
          _agentInfo = 'Linux ${linuxInfo.version}; ${linuxInfo.versionCodename}; ${agentFlag}Messenger ${_packageInfo.version}';
          break;
        case AppConfigDeviceType.windows:
          final windowsInfo = await DeviceInfoPlugin().windowsInfo;
          _deviceInfo = windowsInfo.toMap();
          _agentInfo = 'Windows ${windowsInfo.computerName}; ${agentFlag}Messenger ${_packageInfo.version}';
          break;
        default:
          _deviceInfo = {'name': _deviceType.name};
          _agentInfo = '${_deviceType.name} ${agentFlag}Messenger ${_packageInfo.version}';
      }
    } catch (e) {
      _deviceInfo = {'name': _deviceType.name};
      _agentInfo = '${_deviceType.name} ${agentFlag}Messenger ${_packageInfo.version}';
    }

    logDebug(['_packageInfo =>', packageInfoMap]);
    logDebug(['_deviceInfo =>', _deviceInfo]);
    logDebug(['_agentInfo =>', _agentInfo]);
    //网络库配置
    _netClient = NetClient(
      logger: _deviceType == AppConfigDeviceType.ios ? EasyLogger.stdoutLogger : EasyLogger.printLogger,
      logLevel: EasyLogLevel.warn,
      logTag: '[${_deviceType.name}]',
      host: host,
      bsid: bsid,
      secret: secret,
      binary: true,
      isolate: _deviceType != AppConfigDeviceType.web,
      onCredentials: _saveUserCredentials,
    );
    //安卓保活配置
    if (androidKeepAliveEnable && _deviceType == AppConfigDeviceType.android) {
      final hasPermissions = await FlutterBackground.initialize(
        androidConfig: androidKeepAliveConfig ?? FlutterBackgroundAndroidConfig(notificationTitle: '$agentFlag title', notificationText: '$agentFlag is running...'),
      );
      _androidKeepAlive = hasPermissions;
    } else {
      _androidKeepAlive = false;
    }
    //本地通知配置
    if (notificationsEnable && _deviceType != AppConfigDeviceType.web) {
      _notifications = FlutterLocalNotificationsPlugin();
      await _notifications?.initialize(
        InitializationSettings(
          android: notificationAndroidSettings ?? const AndroidInitializationSettings('@mipmap/ic_launcher'),
          iOS: notificationIosSettings ?? const IOSInitializationSettings(),
          macOS: notificationMacOSSettings ?? const MacOSInitializationSettings(),
          linux: notificationLinuxSettings ?? LinuxInitializationSettings(defaultActionName: agentFlag),
        ),
        onSelectNotification: (String? payload) {
          logInfo(['onSelectNotification =>', payload]);
        },
      );
    } else {
      _notifications = null;
    }
    logInfo(['_androidKeepAlive =>', _androidKeepAlive, '_notifications =>', _notifications != null]);
    //开放平台配置
    AppOpener.register(
      logger: _easyLogger,
      amapSupport: _deviceType == AppConfigDeviceType.ios || _deviceType == AppConfigDeviceType.android,
      amapAndroidKey: amapAndroidKey,
      amapIosKey: amapIosKey,
      amapWebKey: amapWebKey,
      wechatSupport: _deviceType == AppConfigDeviceType.ios || _deviceType == AppConfigDeviceType.android,
      wechatAppId: wechatAppId,
      wechatUniversalLink: wechatUniversalLink,
    );
    //主题配置
    _darkThemeData = darkThemeData ?? _createThemeData(themeColor: themeColor, themeAccentColor: themeAccentColor, brightness: Brightness.dark);
    _lightThemeData = lightThemeData ?? _createThemeData(themeColor: themeColor, themeAccentColor: themeAccentColor, brightness: Brightness.light);
    _sexIconMale = sexIconMale ?? const Icon(Icons.male, color: Colors.blueAccent);
    _sexIconFemale = sexIconFemale ?? const Icon(Icons.female, color: Colors.pinkAccent);
    //初始化状体标志
    _runBackground = false;
    _agreeProtocol = _readAgreeProtocol() ?? false;
    _useDarkTheme = _readThemeBrightness() ?? false;
    _keyboardHeight = _readKeyboardHeight() ?? 258.0;
    //同步状态栏颜色
    SystemChrome.setSystemUIOverlayStyle(systemUiOverlayStyle); //需要在初始化_useDarkTheme之后执行
    //初始化完毕回调
    final user = _readUserCredentials();
    onInited(user?.id, user?.token);
  }

  ///设置是否已经切换到后台
  static void setRunBackground(bool background) async {
    _runBackground = background;
    //安卓前后台切换处理
    if (_androidKeepAlive) {
      if (FlutterBackground.isBackgroundExecutionEnabled != runBackground) {
        try {
          final success = _runBackground ? await FlutterBackground.enableBackgroundExecution() : await FlutterBackground.disableBackgroundExecution();
          logInfo(['setRunBackground =>', '_runBackground:', _runBackground, 'success:', success]);
        } catch (error, stack) {
          logInfo(['setRunBackground =>', '_runBackground:', _runBackground, error, '\n', stack]);
        }
      }
    }
    //清除全部本地通知消息
    if (!_runBackground) {
      _notifications?.cancelAll();
    }
  }

  ///设置当前是否同意协议政策
  static void setAgreeProtocol(bool agree) {
    _agreeProtocol = agree;
    _saveAgreeProtocol(agree);
    for (var element in _onAgreeProtocolChange) {
      element();
    }
  }

  ///设置当前主题是否夜间模式
  static void setBrightness(bool dark) {
    _useDarkTheme = dark;
    SystemChrome.setSystemUIOverlayStyle(systemUiOverlayStyle); //设置_useDarkTheme后再执行
    _saveThemeBrightness(_useDarkTheme);
    for (var element in _onBrightnessChange) {
      element();
    }
  }

  ///设置自定义虚拟键盘高度
  static void setKeyboardHeight(double height) {
    _keyboardHeight = height;
    _saveKeyboardHeight(height);
  }

  ///添加或删除主题日夜间模式变化的监听器
  static void setOnAgreeProtocolChange(void Function() onAgreeProtocolChange, {required bool remove}) {
    if (remove) {
      _onAgreeProtocolChange.remove(onAgreeProtocolChange);
    } else {
      _onAgreeProtocolChange.add(onAgreeProtocolChange);
    }
  }

  ///添加或删除主题日夜间模式变化的监听器
  static void setOnBrightnessChange(void Function() onBrightnessChange, {required bool remove}) {
    if (remove) {
      _onBrightnessChange.remove(onBrightnessChange);
    } else {
      _onBrightnessChange.add(onBrightnessChange);
    }
  }

  //显示本地通知
  static void showNotification({required String title, required String body, required String groupKey, required String androidChannelId, required String androidChannelName, String? androidChannelDescription, String? payload}) {
    _notifications?.show(
      _notificationIdInc++,
      title,
      body,
      NotificationDetails(
        android: AndroidNotificationDetails(androidChannelId, androidChannelName, channelDescription: androidChannelDescription, groupKey: groupKey, importance: Importance.max, priority: Priority.max),
        iOS: IOSNotificationDetails(threadIdentifier: groupKey),
        macOS: MacOSNotificationDetails(threadIdentifier: groupKey),
        linux: LinuxNotificationDetails(defaultActionName: groupKey),
      ),
      payload: payload,
    );
  }

  ///自定义键盘插入内容
  static void insertCustomKeyboardText(FocusNode focusNode, TextEditingController editingController, String text, {int? maxLength}) {
    if (isMobileDevice) {
      if (!focusNode.hasFocus) return; //避免异常
    } else {
      focusNode.requestFocus();
    }
    int baseOffset = editingController.selection.baseOffset;
    int extentOffset = editingController.selection.extentOffset;
    String value = editingController.text.replaceRange(baseOffset, extentOffset, text);
    if (maxLength != null && value.length > maxLength) return; //超长了
    TextSelection selection = TextSelection(baseOffset: baseOffset + text.length, extentOffset: baseOffset + text.length);
    editingController.value = editingController.value.copyWith(text: value, selection: selection); //必须用这种方式同时设置text和selection，否则会成-1，-1
  }

  ///自定义键盘删除内容
  static void deleteCustomKeyboardText(FocusNode focusNode, TextEditingController editingController) {
    if (isMobileDevice) {
      if (!focusNode.hasFocus) return; //避免异常
    } else {
      focusNode.requestFocus();
    }
    int baseOffset = editingController.selection.baseOffset;
    int extentOffset = editingController.selection.extentOffset;
    if (extentOffset - baseOffset > 0) {
      String beforeText = editingController.selection.textBefore(editingController.text);
      String afterText = editingController.selection.textAfter(editingController.text);
      TextSelection selection = TextSelection(baseOffset: beforeText.length, extentOffset: beforeText.length);
      editingController.value = editingController.value.copyWith(text: '$beforeText$afterText', selection: selection); //必须用这种方式同时设置text和selection，否则会成-1，-1
    } else if (baseOffset > 0) {
      String beforeText = editingController.selection.textBefore(editingController.text).characters.skipLast(1).string; //使用扩展字符类characters跳过最后一个【字符簇】
      String afterText = editingController.selection.textAfter(editingController.text);
      TextSelection selection = TextSelection(baseOffset: beforeText.length, extentOffset: beforeText.length);
      editingController.value = editingController.value.copyWith(text: '$beforeText$afterText', selection: selection); //必须用这种方式同时设置text和selection，否则会成-1，-1
    }
  }

  ///设备类型
  static AppConfigDeviceType get deviceType => _deviceType;

  ///安装包信息
  static PackageInfo get packageInfo => _packageInfo;

  ///安装包信息
  static Map<String, dynamic> get packageInfoMap => {'appName': _packageInfo.appName, 'buildNumber': _packageInfo.buildNumber, 'buildSignature': _packageInfo.buildSignature, 'packageName': _packageInfo.packageName, 'version': _packageInfo.version};

  ///设备信息
  static Map<String, dynamic> get deviceInfo => _deviceInfo;

  ///浏览器agent
  static String get agentInfo => _agentInfo;

  ///网络客户端
  static NetClient get client => _netClient;

  ///当前状态的主题
  static ThemeData get themeData => _useDarkTheme ? _darkThemeData : _lightThemeData;

  ///夜间模式的主题
  static ThemeData get darkThemeData => _darkThemeData;

  ///日间模式的主题
  static ThemeData get lightThemeData => _lightThemeData;

  ///当前状态的状态栏
  static SystemUiOverlayStyle get systemUiOverlayStyle => _useDarkTheme ? darkSystemUiOverlayStyle : lightSystemUiOverlayStyle;

  ///夜间模式的状态栏
  static SystemUiOverlayStyle get darkSystemUiOverlayStyle => SystemUiOverlayStyle.light.copyWith(statusBarColor: Colors.transparent);

  ///日间模式的状态栏
  static SystemUiOverlayStyle get lightSystemUiOverlayStyle => SystemUiOverlayStyle.dark.copyWith(statusBarColor: Colors.transparent);

  ///获取工具栏高度
  static double get toolbarHeight => themeData.appBarTheme.toolbarHeight ?? kToolbarHeight;

  ///是否已经切换到后台
  static bool get runBackground => _runBackground;

  ///是否已同意协议政策
  static bool get agreeProtocol => _agreeProtocol;

  ///是否已同意协议政策
  static double get keyboardHeight => _keyboardHeight;

  ///是否为移动设备
  static bool get isMobileDevice => _deviceType == AppConfigDeviceType.android || _deviceType == AppConfigDeviceType.ios;

  ///是否为苹果设备
  static bool get isAppleDevice => _deviceType == AppConfigDeviceType.ios || _deviceType == AppConfigDeviceType.macOS;

  ///性别标志[sex]对应的图标
  static Icon getUserSexIcon(int sex) => sex == Constant.userSexMale ? _sexIconMale : _sexIconFemale;

  ///获取用户建议的未设置字段数量
  static int countUserNotSettings() => (AppConfig.client.user.no.isEmpty ? 1 : 0) + (AppConfig.client.user.pwd.isEmpty ? 1 : 0) + (AppConfig.client.user.phone.isEmpty ? 1 : 0);

  ///清除当前用户登录凭据
  static void clearUserCredentials() => _saveUserCredentials('', null);

  ///格式化附件链接地址
  static String formatAttachUrl(String url) {
    if (url.startsWith('http://') || url.startsWith('https://')) {
      return url;
    } else {
      return '${_netClient.host}${url.startsWith('/') ? '' : '/'}$url';
    }
  }

  ///格式化标志连接地址
  static String formatIdentUrl({User? user, Team? team}) {
    if (user != null) {
      return 'user://${user.id.toHexString()}';
    } else if (team != null) {
      return 'team://${team.id.toHexString()}';
    } else {
      return 'unknow://';
    }
  }

  ///logTrace
  static void logTrace(List<dynamic> args) => _easyLogger.logTrace(args);

  ///logDebug
  static void logDebug(List<dynamic> args) => _easyLogger.logDebug(args);

  ///logInfo
  static void logInfo(List<dynamic> args) => _easyLogger.logInfo(args);

  ///logWarn
  static void logWarn(List<dynamic> args) => _easyLogger.logWarn(args);

  ///logError
  static void logError(List<dynamic> args) => _easyLogger.logError(args);

  /* ******************************** private static functions ******************************** */
  static void _saveAgreeProtocol(bool agree) => _sharedPrefs?.setBool('agreement', agree);

  static bool? _readAgreeProtocol() => _sharedPrefs?.getBool('agreement');

  static void _saveThemeBrightness(bool dark) => _sharedPrefs?.setBool('brightness', dark);

  static bool? _readThemeBrightness() => _sharedPrefs?.getBool('brightness');

  static void _saveKeyboardHeight(double height) => _sharedPrefs?.setDouble('keyboardHeight', height);

  static double? _readKeyboardHeight() => _sharedPrefs?.getDouble('keyboardHeight');

  static void _saveUserCredentials(String nick, String? credentials) => _sharedPrefs?.setString('credentials', credentials ?? '');

  static User? _readUserCredentials() => NetClient.decryptCredentials(_sharedPrefs?.getString('credentials') ?? '', _netClient.secret);

  static ThemeData _createThemeData({required MaterialColor themeColor, required MaterialAccentColor themeAccentColor, required Brightness brightness}) {
    final appBarBackgroundColor = brightness == Brightness.light ? Colors.grey[50] : Colors.grey[850];
    final appBarForegroundColor = brightness == Brightness.light ? Colors.grey[850] : Colors.grey[50];
    final tabBarSelectedLabelColor = brightness == Brightness.light ? themeColor[700] : themeAccentColor;
    final tabBarUnselectedLabelColor = brightness == Brightness.light ? Colors.grey[850] : Colors.grey[50];
    final scaffoldBackgroundColor = brightness == Brightness.light ? Colors.grey[50] : Colors.grey[850];
    final bottomNavigationBarBackgroundColor = brightness == Brightness.light ? Colors.grey[50] : Colors.grey[850];
    final cardColor = brightness == Brightness.light ? const Color(0xFFFFFFFF) : const Color(0xFF353535);
    final dividerColor = brightness == Brightness.light ? const Color(0xFFEAEAEA) : const Color(0xFF444444);
    final inputHintColor = brightness == Brightness.light ? Colors.grey[400] : Colors.grey[600];
    return ThemeData(
      platform: TargetPlatform.android,
      pageTransitionsTheme: const PageTransitionsTheme(builders: {
        TargetPlatform.android: CupertinoPageTransitionsBuilder(),
        TargetPlatform.iOS: CupertinoPageTransitionsBuilder(),
        TargetPlatform.macOS: CupertinoPageTransitionsBuilder(),
        TargetPlatform.linux: CupertinoPageTransitionsBuilder(),
        TargetPlatform.windows: CupertinoPageTransitionsBuilder(),
        TargetPlatform.fuchsia: CupertinoPageTransitionsBuilder(),
      }),
      brightness: brightness,
      primarySwatch: themeColor,
      scaffoldBackgroundColor: scaffoldBackgroundColor,
      cardColor: cardColor,
      textTheme: const TextTheme(
        headline1: TextStyle(fontSize: 96),
        headline2: TextStyle(fontSize: 60),
        headline3: TextStyle(fontSize: 48),
        headline4: TextStyle(fontSize: 34),
        headline5: TextStyle(fontSize: 24),
        headline6: TextStyle(fontSize: 20),
        subtitle1: TextStyle(fontSize: 17), //程序中调用了
        subtitle2: TextStyle(fontSize: 15),
        bodyText1: TextStyle(fontSize: 16), //程序中调用了
        bodyText2: TextStyle(fontSize: 14), //程序中调用了
        caption: TextStyle(fontSize: 12), //程序中调用了
        button: TextStyle(fontSize: 14),
        overline: TextStyle(fontSize: 10),
      ),
      iconTheme: IconThemeData(size: iconGlobalSizeLarge, color: appBarForegroundColor),
      appBarTheme: AppBarTheme(
        elevation: 0,
        centerTitle: true,
        backgroundColor: appBarBackgroundColor,
        foregroundColor: appBarForegroundColor,
        titleTextStyle: TextStyle(fontSize: 18, color: appBarForegroundColor, fontWeight: FontWeight.bold),
        iconTheme: IconThemeData(size: iconGlobalSizeLarge, color: appBarForegroundColor),
        toolbarHeight: 50,
      ),
      tabBarTheme: TabBarTheme(
        indicatorSize: TabBarIndicatorSize.label,
        labelColor: tabBarSelectedLabelColor,
        unselectedLabelColor: tabBarUnselectedLabelColor,
        labelStyle: const TextStyle(fontSize: 16, fontWeight: FontWeight.bold),
        unselectedLabelStyle: const TextStyle(fontSize: 16, fontWeight: FontWeight.bold),
      ),
      bottomAppBarTheme: BottomAppBarTheme(
        elevation: 0,
        color: appBarBackgroundColor,
      ),
      bottomNavigationBarTheme: BottomNavigationBarThemeData(
        type: BottomNavigationBarType.fixed,
        elevation: 0,
        selectedLabelStyle: const TextStyle(fontSize: 14),
        unselectedLabelStyle: const TextStyle(fontSize: 14),
        backgroundColor: bottomNavigationBarBackgroundColor,
      ),
      popupMenuTheme: const PopupMenuThemeData(
        shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(8))),
      ),
      elevatedButtonTheme: ElevatedButtonThemeData(
        style: ElevatedButton.styleFrom(
          elevation: 0,
          shape: const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(8))),
          minimumSize: const Size(double.infinity, 50),
          textStyle: const TextStyle(fontSize: 17),
        ),
      ),
      inputDecorationTheme: InputDecorationTheme(
        border: const OutlineInputBorder(borderRadius: BorderRadius.all(Radius.circular(8))),
        contentPadding: const EdgeInsets.all(16),
        hintStyle: TextStyle(fontSize: 14, color: inputHintColor),
      ),
      checkboxTheme: CheckboxThemeData(
        shape: const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(2))),
        fillColor: MaterialStateProperty.all(themeColor),
      ),
      dividerTheme: DividerThemeData(
        space: 0.5,
        color: dividerColor,
      ),
    );
  }
}
