import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_font_icons/flutter_font_icons.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_opener.dart';
import 'app_router.dart';
import 'widget/widget_dialog.dart';

///
/// 登录注册页类型
///
enum AppLoginPageType {
  ///登录主页（账号密码登录）
  normal,

  ///注册新账号
  register,

  ///重置密码
  forgetPwd,

  ///验证码登录
  phoneCode,
}

///
///登录注册页
///
class AppLogin extends StatefulWidget {
  ///页面类型
  final AppLoginPageType type;

  const AppLogin({Key? key, required this.type}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _AppLoginState();
}

class _AppLoginState extends State<AppLogin> {
  static const _thirdBtnSize = 50.0;
  static const _thirdBtnRadius = 8.0;
  static const _sendCodeBtnFontSize = 17.0;

  final _appOpener = AppOpener();
  final _noController = TextEditingController(text: '');
  final _pwdController = TextEditingController(text: '');
  final _phoneController = TextEditingController(text: '');
  final _codeController = TextEditingController(text: '');

  bool _isNoEmpty = true;
  bool _isPwdEmpty = true;
  bool _isPhoneEmpty = true;
  bool _isCodeEmpty = true;
  bool _pwdObscure = true;

  Timer? _sendCodeTimer;
  int _sendCodeWait = 0;

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['AppLogin.initState()', widget.type]);
    AppConfig.setOnAgreeProtocolChange(_onAgreeProtocolChange, remove: false);
    _appOpener.init(onWechatLoginOk: _onWechatLoginAuthOk);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['AppLogin.dispose()', widget.type]);
    AppConfig.setOnAgreeProtocolChange(_onAgreeProtocolChange, remove: true);
    _appOpener.dispose();
    _noController.dispose();
    _pwdController.dispose();
    _phoneController.dispose();
    _codeController.dispose();
    _sendCodeTimer?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    final queryData = MediaQuery.of(context);
    return Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: _buildAppBar(themeData),
      body: GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: Padding(
          padding: AppConfig.bodyGlobalPaddingMedium.copyWith(bottom: queryData.padding.bottom),
          child: Column(children: _buildBody(themeData)),
        ),
        onTap: () => FocusScope.of(context).unfocus(),
      ),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    switch (widget.type) {
      case AppLoginPageType.normal:
        return AppBar(
          backgroundColor: themeData.scaffoldBackgroundColor,
          title: Text(AppLocalizations.appLoginNormalTitle),
        );
      case AppLoginPageType.register:
        return AppBar(
          backgroundColor: themeData.scaffoldBackgroundColor,
          title: Text(AppLocalizations.appLoginRegisterTitle),
        );
      case AppLoginPageType.forgetPwd:
        return AppBar(
          backgroundColor: themeData.scaffoldBackgroundColor,
          title: Text(AppLocalizations.appLoginForgetPwdTitle),
        );
      case AppLoginPageType.phoneCode:
        return AppBar(
          backgroundColor: themeData.scaffoldBackgroundColor,
          title: Text(AppLocalizations.appLoginPhoneCodeTitle),
        );
    }
  }

  List<Widget> _buildBody(ThemeData themeData) {
    final List<Widget> children;
    switch (widget.type) {
      case AppLoginPageType.normal:
        children = _buildNormalBody(themeData);
        break;
      case AppLoginPageType.register:
        children = _buildRegisterBody(themeData);
        break;
      case AppLoginPageType.forgetPwd:
        children = _buildForgetPwdBody(themeData);
        break;
      case AppLoginPageType.phoneCode:
        children = _buildPhoneCodeBody(themeData);
        break;
    }
    children.add(widget.type == AppLoginPageType.normal ? AppConfig.cardGlobalSpaceMedium : const Spacer());
    children.add(_buildProtocols(alert: false));
    children.add(AppConfig.cardGlobalSpaceMedium);
    return children;
  }

  List<Widget> _buildNormalBody(ThemeData themeData) {
    return [
      _buildFieldNo(register: false),
      AppConfig.cardGlobalSpaceLarge,
      _buildFieldPwd(forgetPwd: false),
      AppConfig.cardGlobalSpaceLarge,
      ElevatedButton(
        onPressed: _isNoEmpty || _isPwdEmpty ? null : _onLoginByNoPwdSureBtnPressed,
        child: Text(AppLocalizations.appLoginNormalBtnSure),
      ),
      Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          TextButton(
            child: Text(AppLocalizations.appLoginForgetPwdTitle),
            onPressed: () => AppRouter.pushAppLoginForgetPwdPage(context),
          ),
          TextButton(
            child: Text(AppLocalizations.appLoginRegisterTitle),
            onPressed: () => AppRouter.pushAppLoginRegisterPage(context),
          ),
        ],
      ),
      const Spacer(),
      Text(AppLocalizations.appLoginNormalThirdMethod, style: themeData.textTheme.caption),
      AppConfig.cardGlobalSpaceMedium,
      Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          SizedBox(
            width: _thirdBtnSize,
            height: _thirdBtnSize,
            child: MaterialButton(
              elevation: 0,
              padding: EdgeInsets.zero,
              color: AppConfig.isAppleDevice ? (themeData.brightness == Brightness.light ? Colors.black : Colors.white) : Colors.green[700],
              shape: const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(_thirdBtnRadius))),
              onPressed: AppConfig.isAppleDevice ? _onLoginByAppleBtnPressed : _onLoginByWechatBtnPressed,
              child: Icon(
                AppConfig.isAppleDevice ? FontAwesome.apple : FontAwesome.wechat,
                color: AppConfig.isAppleDevice ? (themeData.brightness == Brightness.light ? Colors.white : Colors.black) : Colors.white,
              ),
            ),
          ),
          const SizedBox(width: _thirdBtnSize / 2),
          SizedBox(
            width: _thirdBtnSize,
            height: _thirdBtnSize,
            child: MaterialButton(
              elevation: 0,
              padding: EdgeInsets.zero,
              color: Colors.blueGrey,
              shape: const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(_thirdBtnRadius))),
              child: const Icon(FontAwesome.mobile_phone, color: Colors.white, size: 32),
              onPressed: () => AppRouter.pushAppLoginPhoneCodePage(context),
            ),
          ),
        ],
      ),
    ];
  }

  List<Widget> _buildRegisterBody(ThemeData themeData) {
    return [
      _buildFieldNo(register: true),
      AppConfig.cardGlobalSpaceMedium,
      _buildFieldPwd(forgetPwd: false),
      AppConfig.cardGlobalSpaceMedium,
      _buildFieldPhone(),
      AppConfig.cardGlobalSpaceMedium,
      _buildFieldCode(),
      AppConfig.cardGlobalSpaceMedium,
      ElevatedButton(
        onPressed: _isNoEmpty || _isPwdEmpty || _isPhoneEmpty || _isCodeEmpty ? null : () => _onLoginByPhoneSureBtnPressed(no: true, pwd: true),
        child: Text(AppLocalizations.appLoginRegisterBtnSure),
      ),
    ];
  }

  List<Widget> _buildForgetPwdBody(ThemeData themeData) {
    return [
      _buildFieldPhone(),
      AppConfig.cardGlobalSpaceLarge,
      _buildFieldCode(),
      AppConfig.cardGlobalSpaceLarge,
      _buildFieldPwd(forgetPwd: true),
      AppConfig.cardGlobalSpaceLarge,
      ElevatedButton(
        onPressed: _isPhoneEmpty || _isCodeEmpty || _isPwdEmpty ? null : () => _onLoginByPhoneSureBtnPressed(no: false, pwd: true),
        child: Text(AppLocalizations.appLoginForgetPwdBtnSure),
      ),
    ];
  }

  List<Widget> _buildPhoneCodeBody(ThemeData themeData) {
    return [
      _buildFieldPhone(),
      AppConfig.cardGlobalSpaceLarge,
      _buildFieldCode(),
      AppConfig.cardGlobalSpaceLarge,
      ElevatedButton(
        onPressed: _isPhoneEmpty || _isCodeEmpty ? null : () => _onLoginByPhoneSureBtnPressed(no: false, pwd: false),
        child: Text(AppLocalizations.appLoginPhoneCodeBtnSure),
      ),
    ];
  }

  Widget _buildFieldNo({required bool register}) {
    return TextFormField(
      controller: _noController,
      keyboardType: TextInputType.emailAddress,
      textInputAction: TextInputAction.next,
      maxLength: Validator.noLenMax,
      decoration: InputDecoration(
        prefixIcon: const Icon(Icons.account_circle),
        labelText: AppLocalizations.appLoginFieldLabelNo,
        hintText: register ? AppLocalizations.appLoginFieldHintNewNo : AppLocalizations.appLoginFieldHintNo,
        counterText: '',
      ),
      onChanged: (value) {
        final empty = value.trim() == '';
        if (_isNoEmpty != empty) {
          if (mounted) {
            setState(() {
              _isNoEmpty = empty;
            });
          }
        }
      },
    );
  }

  Widget _buildFieldPwd({required bool forgetPwd}) {
    return TextFormField(
      obscureText: _pwdObscure,
      controller: _pwdController,
      keyboardType: TextInputType.visiblePassword,
      textInputAction: TextInputAction.next,
      maxLength: Validator.pwdLenMax,
      decoration: InputDecoration(
        prefixIcon: const Icon(Icons.lock),
        labelText: forgetPwd ? AppLocalizations.appLoginFieldLabelNewPwd : AppLocalizations.appLoginFieldLabelPwd,
        hintText: forgetPwd ? AppLocalizations.appLoginFieldHintNewPwd : AppLocalizations.appLoginFieldHintPwd,
        counterText: '',
        suffixIcon: IconButton(
          icon: Icon(_pwdObscure ? Icons.visibility : Icons.visibility_off),
          onPressed: () {
            if (mounted) {
              setState(() {
                _pwdObscure = !_pwdObscure;
              });
            }
          },
        ),
      ),
      onChanged: (value) {
        final empty = value.trim() == '';
        if (_isPwdEmpty != empty) {
          if (mounted) {
            setState(() {
              _isPwdEmpty = empty;
            });
          }
        }
      },
    );
  }

  Widget _buildFieldPhone() {
    return TextFormField(
      controller: _phoneController,
      keyboardType: TextInputType.phone,
      textInputAction: TextInputAction.next,
      maxLength: Validator.phoneLenMax,
      decoration: InputDecoration(
        prefixIcon: const Icon(Icons.phone_android),
        labelText: AppLocalizations.appLoginFieldLabelPhone,
        hintText: AppLocalizations.appLoginFieldHintPhone,
        counterText: '',
      ),
      onChanged: (value) {
        final empty = value.trim() == '';
        if (_isPhoneEmpty != empty) {
          if (mounted) {
            setState(() {
              _isPhoneEmpty = empty;
            });
          }
        }
      },
    );
  }

  Widget _buildFieldCode() {
    return TextFormField(
      controller: _codeController,
      keyboardType: TextInputType.phone,
      textInputAction: TextInputAction.next,
      maxLength: Validator.randcodeLen,
      decoration: InputDecoration(
        prefixIcon: const Icon(Icons.verified_user),
        labelText: AppLocalizations.appLoginFieldLabelCode,
        hintText: AppLocalizations.appLoginFieldHintCode,
        counterText: '',
        suffixIcon: TextButton(
          onPressed: _isPhoneEmpty || _sendCodeWait > 0 ? null : _onSendRandCodeBtnPressed,
          child: Text(_sendCodeWait > 0 ? '$_sendCodeWait S' : AppLocalizations.appLoginFieldBtnSendCode, style: const TextStyle(fontSize: _sendCodeBtnFontSize)),
        ),
      ),
      onChanged: (value) {
        final empty = value.trim() == '';
        if (_isCodeEmpty != empty) {
          if (mounted) {
            setState(() {
              _isCodeEmpty = empty;
            });
          }
        }
      },
    );
  }

  Widget _buildProtocols({required bool alert}) {
    final themeData = Theme.of(context);
    if (alert) {
      final textStyle = themeData.textTheme.subtitle1;
      final linkStyle = themeData.textTheme.subtitle1?.copyWith(color: themeData.colorScheme.primary);
      return RichText(
        textAlign: TextAlign.center,
        text: TextSpan(
          children: [
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: Text(AppLocalizations.appLoginProtoReadAgreeAlert, style: textStyle),
            ),
            const TextSpan(text: '\n'),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: InkWell(child: Text(AppLocalizations.appLoginProtoBtnUserText, style: linkStyle), onTap: () => AppRouter.pushProtocolAccountPage(context)),
            ),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: Text(AppLocalizations.appLoginProtoAnd, style: textStyle),
            ),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: InkWell(child: Text(AppLocalizations.appLoginProtoBtnProvText, style: linkStyle), onTap: () => AppRouter.pushProtocolPrivacyPage(context)),
            ),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: Text('?', style: textStyle),
            ),
          ],
        ),
      );
    } else {
      final textStyle = themeData.textTheme.bodyText2;
      final linkStyle = themeData.textTheme.bodyText2?.copyWith(color: themeData.colorScheme.primary);
      return RichText(
        text: TextSpan(
          children: [
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: Checkbox(
                value: AppConfig.agreeProtocol,
                onChanged: (value) {
                  if (mounted) {
                    setState(() {
                      AppConfig.setAgreeProtocol(!AppConfig.agreeProtocol);
                    });
                  }
                },
              ),
            ),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: InkWell(
                child: Text(AppLocalizations.appLoginProtoReadAgreeBody, style: textStyle),
                onTap: () {
                  if (mounted) {
                    setState(() {
                      AppConfig.setAgreeProtocol(!AppConfig.agreeProtocol);
                    });
                  }
                },
              ),
            ),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: InkWell(child: Text(AppLocalizations.appLoginProtoBtnUserText, style: linkStyle), onTap: () => AppRouter.pushProtocolAccountPage(context)),
            ),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: Text(AppLocalizations.appLoginProtoAnd, style: textStyle),
            ),
            WidgetSpan(
              alignment: PlaceholderAlignment.middle,
              child: InkWell(child: Text(AppLocalizations.appLoginProtoBtnProvText, style: linkStyle), onTap: () => AppRouter.pushProtocolPrivacyPage(context)),
            ),
          ],
        ),
      );
    }
  }

  Future<bool> _forceAgreeProtocol() async {
    if (AppConfig.agreeProtocol) return true;
    final agree = await WidgetDialog.alert(
      context,
      bodyWidget: _buildProtocols(alert: true),
      leftText: AppLocalizations.appLoginProtoAlertBtnNo,
      rightText: AppLocalizations.appLoginProtoAlertBtnYes,
      rightColor: Theme.of(context).colorScheme.secondary,
    );
    if (agree == null || !agree) return false;
    if (mounted) {
      setState(() {
        AppConfig.setAgreeProtocol(true);
      });
    }
    return true;
  }

  void _onSendRandCodeBtnPressed() async {
    SmartDialog.showLoading(msg: AppLocalizations.appLoginSendLoading);
    final result = await AppConfig.client.sendRandcode(phone: _phoneController.text.trim());
    SmartDialog.dismiss();
    if (result.ok) {
      if (mounted) {
        setState(() {
          _sendCodeWait = 60; //开始倒计时
        });
      }
      _sendCodeTimer?.cancel();
      _sendCodeTimer = Timer.periodic(const Duration(seconds: 1), (timer) {
        if (_sendCodeWait > 0) {
          if (mounted) {
            setState(() {
              _sendCodeWait--;
            });
          }
        } else {
          _sendCodeTimer?.cancel();
          _sendCodeTimer = null;
        }
      });
    }
    SmartDialog.showToast(result.desc);
  }

  void _onLoginByAppleBtnPressed() async {
    if (!await _forceAgreeProtocol()) return;
    _appOpener.startAppleLogin(_onAppleLoginAuthOk);
  }

  void _onLoginByWechatBtnPressed() async {
    if (!await _forceAgreeProtocol()) return;
    if (await AppOpener.isWechatInstalled()) {
      _appOpener.startWechatLogin();
    } else {
      SmartDialog.showToast(AppLocalizations.appLoginNormalNoWechatHint);
    }
  }

  void _onLoginByNoPwdSureBtnPressed() async {
    if (!await _forceAgreeProtocol()) return;
    SmartDialog.showLoading(msg: AppLocalizations.appLoginSureLoading);
    final result = await AppConfig.client.loginByNoPwd(no: _noController.text.trim(), pwd: _pwdController.text.trim());
    SmartDialog.dismiss();
    if (result.ok) {
      if (!mounted) return;
      AppRouter.pushAppHomePage(context);
    } else {
      SmartDialog.showToast(result.desc);
    }
  }

  void _onLoginByPhoneSureBtnPressed({required bool no, required bool pwd}) async {
    if (!await _forceAgreeProtocol()) return;
    SmartDialog.showLoading(msg: AppLocalizations.appLoginSureLoading);
    final result = await AppConfig.client.loginByPhone(phone: _phoneController.text.trim(), code: _codeController.text.trim(), no: no ? _noController.text.trim() : null, pwd: pwd ? _pwdController.text.trim() : null);
    SmartDialog.dismiss();
    if (result.ok) {
      if (!mounted) return;
      AppRouter.pushAppHomePage(context);
    } else {
      SmartDialog.showToast(result.desc);
    }
  }

  void _onAppleLoginAuthOk(String appleUid, String appleUname, String authorizationCode, String? identityToken) async {
    SmartDialog.showLoading(msg: AppLocalizations.appLoginSureLoading);
    final result = await AppConfig.client.loginByApple(appleUid: appleUid, appleUname: appleUname, authorizationCode: authorizationCode, identityToken: identityToken);
    SmartDialog.dismiss();
    if (result.ok) {
      if (!mounted) return;
      AppRouter.pushAppHomePage(context);
    } else {
      SmartDialog.showToast(result.desc);
    }
  }

  void _onWechatLoginAuthOk(String wechatCode) async {
    SmartDialog.showLoading(msg: AppLocalizations.appLoginSureLoading);
    final result = await AppConfig.client.loginByWechat(wechatCode: wechatCode);
    SmartDialog.dismiss();
    if (result.ok) {
      if (!mounted) return;
      AppRouter.pushAppHomePage(context);
    } else {
      SmartDialog.showToast(result.desc);
    }
  }

  void _onAgreeProtocolChange() {
    if (mounted) {
      setState(() {});
    }
  }
}
