import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'widget/widget_divider.dart';

///
///联系人编辑器类型
///
enum ContactEditorType {
  ///好友的备注名
  usershipAlias,

  ///发起好友申请
  usershipApply,

  ///群组名称
  teamNick,

  ///群组公告
  teamDesc,

  ///群组权限
  teamSettings,

  ///我的群昵称
  teamshipAlias,

  ///自定义字段
  customField,
}

///
///联系人编辑器
///
class ContactEditor extends StatefulWidget {
  ///页面标题
  final String fieldTitle;

  ///编辑器类型
  final ContactEditorType editorType;

  ///编辑目标对象 [UserShip] 或 [TeamShip]
  final Object targetShip;

  ///输入框的标签
  final String? inputLabel;

  const ContactEditor({Key? key, required this.fieldTitle, required this.editorType, required this.targetShip, this.inputLabel}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _UsermeEditorState();
}

class _UsermeEditorState extends State<ContactEditor> {
  final _valueController = TextEditingController();
  bool _hasTitlePreffix = true;
  bool _hasBodyPadding = true;

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['ContactEditor.initState()', widget.editorType]);
    if (widget.editorType == ContactEditorType.teamSettings) {
      AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: false);
      AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: false);
      AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: false);
    }
    _initInputFieldValue();
  }

  @override
  void dispose() {
    AppConfig.logDebug(['ContactEditor.dispose()', widget.editorType]);
    if (widget.editorType == ContactEditorType.teamSettings) {
      AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: true);
      AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: true);
      AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: true);
    }
    _valueController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      appBar: _buildAppBar(themeData),
      body: _buildBody(themeData),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    return AppBar(
      title: Text('${_hasTitlePreffix ? AppLocalizations.contactEditorTitlePreffix : ''}${_hasTitlePreffix ? widget.fieldTitle.toLowerCase() : widget.fieldTitle}'),
      actions: [_buildSaveButton(themeData)],
    );
  }

  Widget _buildBody(ThemeData themeData) {
    return SafeArea(
      left: _hasBodyPadding,
      right: _hasBodyPadding,
      child: GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: ListView(
          physics: AppConfig.globalScrollPhysics,
          padding: _hasBodyPadding ? AppConfig.bodyGlobalPaddingMedium : EdgeInsets.zero,
          children: _buildBodyField(themeData),
        ),
        onTap: () {
          FocusScope.of(context).unfocus();
        },
      ),
    );
  }

  Widget _buildSaveButton(ThemeData themeData) {
    switch (widget.editorType) {
      case ContactEditorType.teamSettings:
        return const SizedBox();
      default:
        return TextButton(
          onPressed: _onSaveBtnPressed,
          child: Text(AppLocalizations.contactEditorActionBtnSave),
        );
    }
  }

  List<Widget> _buildBodyField(ThemeData themeData) {
    switch (widget.editorType) {
      case ContactEditorType.usershipAlias:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case ContactEditorType.usershipApply:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.mediumTextLenMax,
            minLines: 3,
            maxLines: 5,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case ContactEditorType.teamNick:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case ContactEditorType.teamDesc:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.mediumTextLenMax,
            minLines: 3,
            maxLines: 5,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case ContactEditorType.teamSettings:
        final team = AppConfig.client.getTeam(_teamship.rid);
        final isAdmin = ComTools.isTeamAdmin(team, AppConfig.client.user.id);
        final onText = AppLocalizations.contactEditorTeamOnText;
        final offText = AppLocalizations.contactEditorTeamOffText;
        return [
          Ink(
            color: themeData.cardColor,
            child: Column(
              children: [
                WidgetDivider.forFull,
                ListTile(
                  contentPadding: AppConfig.infoListTileItemPadding.copyWith(right: isAdmin ? AppConfig.infoListTileItemPadding.right / 2 : AppConfig.infoListTileItemPadding.left),
                  leading: Text(AppLocalizations.contactEditorTeamByfind, style: themeData.textTheme.subtitle1),
                  trailing: isAdmin ? Switch(value: team.byfind, onChanged: (value) => _onTeamSettingsByFindItemTap()) : Text(team.byfind ? onText : offText),
                  onTap: isAdmin ? _onTeamSettingsByFindItemTap : null,
                ),
                WidgetDivider.forListTileLeadingNone,
                ListTile(
                  contentPadding: AppConfig.infoListTileItemPadding.copyWith(right: isAdmin ? AppConfig.infoListTileItemPadding.right / 2 : AppConfig.infoListTileItemPadding.left),
                  leading: Text(AppLocalizations.contactEditorTeamBycode, style: themeData.textTheme.subtitle1),
                  trailing: isAdmin ? Switch(value: team.bycode, onChanged: (value) => _onTeamSettingsByCodeItemTap()) : Text(team.bycode ? onText : offText),
                  onTap: isAdmin ? _onTeamSettingsByCodeItemTap : null,
                ),
                WidgetDivider.forListTileLeadingNone,
                ListTile(
                  contentPadding: AppConfig.infoListTileItemPadding.copyWith(right: isAdmin ? AppConfig.infoListTileItemPadding.right / 2 : AppConfig.infoListTileItemPadding.left),
                  leading: Text(AppLocalizations.contactEditorTeamBycard, style: themeData.textTheme.subtitle1),
                  trailing: isAdmin ? Switch(value: team.bycard, onChanged: (value) => _onTeamSettingsByCardItemTap()) : Text(team.bycard ? onText : offText),
                  onTap: isAdmin ? _onTeamSettingsByCardItemTap : null,
                ),
                WidgetDivider.forListTileLeadingNone,
                ListTile(
                  contentPadding: AppConfig.infoListTileItemPadding.copyWith(right: isAdmin ? AppConfig.infoListTileItemPadding.right / 2 : AppConfig.infoListTileItemPadding.left),
                  leading: Text(AppLocalizations.contactEditorTeamByteam, style: themeData.textTheme.subtitle1),
                  trailing: isAdmin ? Switch(value: team.byteam, onChanged: (value) => _onTeamSettingsByTeamItemTap()) : Text(team.byteam ? onText : offText),
                  onTap: isAdmin ? _onTeamSettingsByTeamItemTap : null,
                ),
                WidgetDivider.forFull,
              ],
            ),
          ),
          AppConfig.cardGlobalSpaceMedium,
          Ink(
            color: themeData.cardColor,
            child: Column(
              children: [
                WidgetDivider.forFull,
                ListTile(
                  contentPadding: AppConfig.infoListTileItemPadding.copyWith(right: isAdmin ? AppConfig.infoListTileItemPadding.right / 2 : AppConfig.infoListTileItemPadding.left),
                  leading: Text(AppLocalizations.contactEditorTeamNotice, style: themeData.textTheme.subtitle1),
                  trailing: isAdmin ? Switch(value: team.notice, onChanged: (value) => _onTeamSettingsNoticeItemTap()) : Text(team.notice ? onText : offText),
                  onTap: isAdmin ? _onTeamSettingsNoticeItemTap : null,
                ),
                WidgetDivider.forListTileLeadingNone,
                ListTile(
                  contentPadding: AppConfig.infoListTileItemPadding.copyWith(right: isAdmin ? AppConfig.infoListTileItemPadding.right / 2 : AppConfig.infoListTileItemPadding.left),
                  leading: Text(AppLocalizations.contactEditorTeamSlient, style: themeData.textTheme.subtitle1),
                  trailing: isAdmin ? Switch(value: team.silent, onChanged: (value) => _onTeamSettingsSlientItemTap()) : Text(team.silent ? onText : offText),
                  onTap: isAdmin ? _onTeamSettingsSlientItemTap : null,
                ),
                WidgetDivider.forFull,
              ],
            ),
          ),
        ];
      case ContactEditorType.teamshipAlias:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      default:
        return [
          Text(widget.fieldTitle, style: themeData.textTheme.subtitle1, textAlign: TextAlign.center),
        ];
    }
  }

  void _initInputFieldValue() {
    switch (widget.editorType) {
      case ContactEditorType.usershipAlias:
        _valueController.text = _usership.alias;
        break;
      case ContactEditorType.usershipApply:
        _valueController.text = '';
        _hasTitlePreffix = false;
        break;
      case ContactEditorType.teamNick:
        _valueController.text = AppConfig.client.getTeam(_teamship.rid).nick;
        break;
      case ContactEditorType.teamDesc:
        _valueController.text = AppConfig.client.getTeam(_teamship.rid).desc;
        break;
      case ContactEditorType.teamSettings:
        _hasTitlePreffix = ComTools.isTeamAdmin(AppConfig.client.getTeam(_teamship.rid), AppConfig.client.user.id);
        _hasBodyPadding = false;
        break;
      case ContactEditorType.teamshipAlias:
        _valueController.text = _teamship.alias;
        break;
      default:
        break;
    }
  }

  void _onSaveBtnPressed() async {
    switch (widget.editorType) {
      case ContactEditorType.usershipAlias:
        final ship = UserShipDirty();
        ship.alias = _valueController.text.trim();
        SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
        final result = await AppConfig.client.userShipUpdate(id: _usership.id, fields: ship.data);
        SmartDialog.dismiss();
        SmartDialog.showToast(result.desc);
        if (result.ok) {
          if (!mounted) return;
          Navigator.pop(context);
        }
        break;
      case ContactEditorType.usershipApply:
        SmartDialog.showLoading(msg: AppLocalizations.contactEditorApplyLoading);
        final result = await AppConfig.client.userShipApply(uid: _usership.rid, fid: _usership.fid, from: _usership.from, apply: _valueController.text.trim());
        SmartDialog.dismiss();
        SmartDialog.showToast(result.desc);
        if (result.ok) {
          if (!mounted) return;
          Navigator.pop(context);
        }
        break;
      case ContactEditorType.teamNick:
        final team = TeamDirty();
        team.nick = _valueController.text.trim();
        SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
        final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
        SmartDialog.dismiss();
        SmartDialog.showToast(result.desc);
        if (result.ok) {
          if (!mounted) return;
          Navigator.pop(context);
        }
        break;
      case ContactEditorType.teamDesc:
        final team = TeamDirty();
        team.desc = _valueController.text.trim();
        SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
        final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
        SmartDialog.dismiss();
        SmartDialog.showToast(result.desc);
        if (result.ok) {
          if (!mounted) return;
          Navigator.pop(context);
        }
        break;
      case ContactEditorType.teamshipAlias:
        final ship = TeamShipDirty();
        ship.alias = _valueController.text.trim();
        SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
        final result = await AppConfig.client.teamShipUpdate(id: _teamship.id, fields: ship.data);
        SmartDialog.dismiss();
        SmartDialog.showToast(result.desc);
        if (result.ok) {
          if (!mounted) return;
          Navigator.pop(context);
        }
        break;
      default:
        break;
    }
  }

  void _onTeamSettingsByFindItemTap() async {
    final team = TeamDirty();
    team.byfind = !AppConfig.client.getTeam(_teamship.rid).byfind;
    SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
    final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
  }

  void _onTeamSettingsByCodeItemTap() async {
    final team = TeamDirty();
    team.bycode = !AppConfig.client.getTeam(_teamship.rid).bycode;
    SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
    final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
  }

  void _onTeamSettingsByCardItemTap() async {
    final team = TeamDirty();
    team.bycard = !AppConfig.client.getTeam(_teamship.rid).bycard;
    SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
    final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
  }

  void _onTeamSettingsByTeamItemTap() async {
    final team = TeamDirty();
    team.byteam = !AppConfig.client.getTeam(_teamship.rid).byteam;
    SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
    final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
  }

  void _onTeamSettingsNoticeItemTap() async {
    final team = TeamDirty();
    team.notice = !AppConfig.client.getTeam(_teamship.rid).notice;
    SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
    final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
  }

  void _onTeamSettingsSlientItemTap() async {
    final team = TeamDirty();
    team.silent = !AppConfig.client.getTeam(_teamship.rid).silent;
    SmartDialog.showLoading(msg: AppLocalizations.contactEditorSaveLoading);
    final result = await AppConfig.client.teamUpdate(tid: _teamship.rid, fields: team.data);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
  }

  void _clientCommonListener(EasyPacket packet) {
    if (mounted) {
      setState(() {});
    }
  }

  UserShip get _usership => widget.targetShip as UserShip;

  TeamShip get _teamship => widget.targetShip as TeamShip;
}
