import 'package:amap_flutter_map/amap_flutter_map.dart';
import 'package:amap_flutter_base/amap_flutter_base.dart';
import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:flutter_svg/svg.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';
import 'package:map_launcher/map_launcher.dart' as outermap;

import 'app_config.dart';
import 'app_locale.dart';
import 'app_opener.dart';
import 'widget/widget_divider.dart';
import 'widget/widget_picture.dart';
import 'widget/widget_sheet.dart';

///
///高德地图页类型
///
enum SessionAmapType {
  ///显示定位信息
  locationView,

  ///定位并发送位置分享消息
  locationShare,

  ///实时位置分享
  realtimeShare,
}

///
///高德地图页
///
class SessionAmap extends StatefulWidget {
  //类型
  final SessionAmapType type;

  ///位置消息
  final Location? location;

  const SessionAmap({Key? key, required this.type, this.location})
      : assert(type != SessionAmapType.locationView || location != null),
        super(key: key);

  @override
  State<StatefulWidget> createState() => _SessionAmapSendState();
}

class _SessionAmapSendState extends State<SessionAmap> {
  static const _mapInitZoom = 16.0;
  static const _mapToolLayerPadding = 8.0;
  static const _mapToolLayerBtnGap = 2.0;
  static const _mapToolLayerBtnSize = 32.0;
  static const _mapToolLayerBtnIconSize = 20.0;
  static const _navigatorTailIconSize = 38.0;
  static const _navigatorItemIconSize = 42.0;
  static const _navigatorItemIconRadius = BorderRadius.all(Radius.circular(4));
  static const _navigatorItemInnerGap = 6.0;
  static const _navigatorItemInnerPadding = EdgeInsets.symmetric(horizontal: 4);

  final _appOpener = AppOpener();
  final _scrollController = ScrollController();

  final _pointCenter = Location(); //当前POI搜索中心
  final _pointList = <Location>[]; //当前POI搜索中心搜索结果列表
  int _pointIndex = 0; //当前POI搜索中心搜索结果列表的选中index

  AMapController? _mapController; //地图控制器
  MapType? _mapStyle; //当前地图图层类型
  late double _mapZoomLevel; //当前地图缩放级别
  late LatLng _mapCenterMarker; //当前地图中心位置标记
  bool _mapMoveByBtnTap = false; //触发的onCameraMove()与onCameraMoveEnd()

  final List<outermap.AvailableMap> _outerMapList = []; //外置地图

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['SessionAmap.initState()']);
    if (widget.type == SessionAmapType.locationView) {
      //初始位置
      _pointCenter.updateByJson(widget.location!.toJson());
      _mapZoomLevel = widget.location!.zoomLevel;
      _mapCenterMarker = LatLng(_pointCenter.latitude, _pointCenter.longitude);
      //获取外部地图列表
      _getOuterMapList();
    } else if (widget.type == SessionAmapType.locationShare) {
      AppConfig.client.setOnUserUpdateListener(_onUserUpdateListener, remove: false);
      _appOpener.init(onAmapLocationOk: _onAmapLocationOk);
      //初始位置
      _pointCenter.updateByJson(AppConfig.client.user.location.toJson());
      _mapZoomLevel = _mapInitZoom;
      _mapCenterMarker = LatLng(_pointCenter.latitude, _pointCenter.longitude);
      //搜索POI
      _searchPoiAndMoveTo(latitude: _pointCenter.latitude, longitude: _pointCenter.longitude);
      //定位一次
      _appOpener.startAmapLocation(onceLocation: true);
    }
  }

  @override
  void dispose() {
    AppConfig.logDebug(['SessionAmap.dispose()']);
    if (widget.type == SessionAmapType.locationShare) {
      AppConfig.client.setOnUserUpdateListener(_onUserUpdateListener, remove: true);
      _appOpener.dispose();
    }
    _scrollController.dispose();
    _mapController?.disponse();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    final queryData = MediaQuery.of(context);
    return Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: _buildAppBar(themeData, queryData),
      body: _buildBody(themeData, queryData),
    );
  }

  AppBar _buildAppBar(ThemeData themeData, MediaQueryData queryData) {
    switch (widget.type) {
      case SessionAmapType.locationView:
        return AppBar(
          title: Text(AppLocalizations.sessionAmapTitleLocationView),
        );
      case SessionAmapType.locationShare:
        return AppBar(
          title: Text(AppLocalizations.sessionAmapTitleShareMessage),
          actions: [
            TextButton(
              onPressed: _pointList.isNotEmpty ? _onSendBtnPressed : null,
              child: Text(AppLocalizations.sessionAmapActionBtnSure),
            ),
          ],
        );
      case SessionAmapType.realtimeShare:
        return AppBar(
          title: Text(AppLocalizations.sessionAmapTitleRealtimeShare),
        );
    }
  }

  Widget _buildBody(ThemeData themeData, MediaQueryData queryData) {
    final currMapType = _mapStyle ?? (themeData.brightness == Brightness.dark ? MapType.night : MapType.normal);
    switch (widget.type) {
      case SessionAmapType.locationView:
        return Column(
          children: [
            WidgetDivider.forFull,
            Expanded(
              child: Stack(
                children: [
                  AMapWidget(
                    privacyStatement: const AMapPrivacyStatement(hasContains: true, hasShow: true, hasAgree: true),
                    mapType: currMapType,
                    initialCameraPosition: CameraPosition(target: _mapCenterMarker, zoom: _mapZoomLevel),
                    markers: {
                      Marker(position: _mapCenterMarker, icon: BitmapDescriptor.defaultMarkerWithHue(BitmapDescriptor.hueRed)),
                    },
                    onMapCreated: (AMapController controller) => _mapController = controller,
                  ),
                  _buildToolLayer(themeData),
                ],
              ),
            ),
            WidgetDivider.forFull,
            Container(
              padding: EdgeInsets.only(bottom: queryData.padding.bottom),
              child: ListTile(
                title: Text(_pointCenter.description, overflow: TextOverflow.ellipsis),
                subtitle: Text(_pointCenter.address, style: themeData.textTheme.caption, overflow: TextOverflow.ellipsis),
                trailing: Icon(Icons.assistant_direction, size: _navigatorTailIconSize, color: _outerMapList.isNotEmpty ? themeData.colorScheme.secondary : null),
                onTap: _outerMapList.isNotEmpty ? () => _onNavigateItemTap(themeData) : null,
              ),
            )
          ],
        );
      case SessionAmapType.locationShare:
        return Column(
          children: [
            WidgetDivider.forFull,
            Expanded(
              flex: 3,
              child: Stack(
                children: [
                  AMapWidget(
                    privacyStatement: const AMapPrivacyStatement(hasContains: true, hasShow: true, hasAgree: true),
                    mapType: currMapType,
                    buildingsEnabled: true,
                    tiltGesturesEnabled: false,
                    rotateGesturesEnabled: false,
                    initialCameraPosition: CameraPosition(target: _mapCenterMarker, zoom: _mapZoomLevel),
                    markers: {
                      Marker(position: _mapCenterMarker, icon: BitmapDescriptor.defaultMarkerWithHue(BitmapDescriptor.hueRed)),
                    },
                    onMapCreated: (AMapController controller) => _mapController = controller,
                    onCameraMove: (position) {
                      if (mounted) {
                        setState(() {
                          _mapZoomLevel = position.zoom;
                          _mapCenterMarker = position.target;
                        });
                      }
                    },
                    onCameraMoveEnd: (position) {
                      if (mounted) {
                        setState(() {
                          _mapZoomLevel = position.zoom;
                          _mapCenterMarker = position.target;
                        });
                      }
                      if (_mapMoveByBtnTap) {
                        AppConfig.logDebug(['是由点击了按钮触发的onCameraMoveEnd()事件函数，即_mapMoveByBtnTap 为 $_mapMoveByBtnTap，不执行_searchPoiAndMoveTo(...)操作']);
                        _mapMoveByBtnTap = false;
                      } else {
                        AppConfig.logDebug(['不是由点击按钮触发的onCameraMoveEnd()事件函数，即_mapMoveByBtnTap 为 $_mapMoveByBtnTap，执行了_searchPoiAndMoveTo(...)操作']);
                        _searchPoiAndMoveTo(latitude: position.target.latitude, longitude: position.target.longitude);
                      }
                    },
                  ),
                  _buildToolLayer(themeData),
                ],
              ),
            ),
            WidgetDivider.forFull,
            Container(
              alignment: Alignment.centerLeft,
              color: themeData.scaffoldBackgroundColor,
              padding: AppConfig.bodyGlobalPaddingMedium.copyWith(top: AppConfig.bodyGlobalPaddingMini.top, bottom: AppConfig.bodyGlobalPaddingMini.bottom),
              child: Text(AppLocalizations.sessionAmapPoiListBarTitle, textAlign: TextAlign.left, style: themeData.textTheme.bodyText2),
            ),
            WidgetDivider.forFull,
            Expanded(
              flex: 2,
              child: _pointList.isEmpty
                  ? const Center(child: CircularProgressIndicator())
                  : ListView.builder(
                      padding: EdgeInsets.only(bottom: queryData.padding.top),
                      physics: AppConfig.globalScrollPhysics,
                      controller: _scrollController,
                      itemCount: _pointList.length,
                      itemBuilder: (context, index) {
                        final item = _pointList[index];
                        return Column(
                          children: [
                            ListTile(
                              title: Text(item.description, overflow: TextOverflow.ellipsis),
                              subtitle: Text('${_formatDistance(item.distance)} | ${item.address}', style: themeData.textTheme.caption, overflow: TextOverflow.ellipsis),
                              trailing: index == _pointIndex ? Icon(Icons.check, color: themeData.colorScheme.secondary) : null,
                              onTap: () => _onPoiListItemTap(item, index),
                            ),
                            WidgetDivider.forListTileLeadingNone,
                          ],
                        );
                      },
                    ),
            )
          ],
        );
      case SessionAmapType.realtimeShare:
        return const SizedBox();
    }
  }

  Widget _buildToolLayer(ThemeData themeData) {
    final btnColor = themeData.scaffoldBackgroundColor.withOpacity(0.8);
    return Positioned(
      top: _mapToolLayerPadding,
      right: _mapToolLayerPadding,
      bottom: _mapToolLayerPadding,
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(
            width: _mapToolLayerBtnSize,
            height: _mapToolLayerBtnSize,
            child: MaterialButton(
              padding: EdgeInsets.zero,
              color: btnColor,
              onPressed: () {
                if (mounted) {
                  setState(() {
                    _mapStyle = MapType.navi;
                  });
                }
              },
              child: const Icon(Icons.directions_car_outlined, size: _mapToolLayerBtnIconSize),
            ),
          ),
          const SizedBox(height: _mapToolLayerBtnGap),
          SizedBox(
            width: _mapToolLayerBtnSize,
            height: _mapToolLayerBtnSize,
            child: MaterialButton(
              padding: EdgeInsets.zero,
              color: btnColor,
              onPressed: () {
                if (mounted) {
                  setState(() {
                    _mapStyle = MapType.satellite;
                  });
                }
              },
              child: const Icon(Icons.filter_hdr_outlined, size: _mapToolLayerBtnIconSize),
            ),
          ),
          const SizedBox(height: _mapToolLayerBtnGap),
          SizedBox(
            width: _mapToolLayerBtnSize,
            height: _mapToolLayerBtnSize,
            child: MaterialButton(
              padding: EdgeInsets.zero,
              color: btnColor,
              colorBrightness: themeData.brightness,
              onPressed: () {
                if (mounted) {
                  setState(() {
                    _mapStyle = themeData.brightness == Brightness.dark ? MapType.night : MapType.normal;
                  });
                }
              },
              child: const Icon(Icons.map_outlined, size: _mapToolLayerBtnIconSize),
            ),
          ),
          const Spacer(),
          SizedBox(
            width: _mapToolLayerBtnSize,
            height: _mapToolLayerBtnSize,
            child: MaterialButton(
              padding: EdgeInsets.zero,
              color: btnColor,
              onPressed: _moveCameraByZoomIn,
              child: const Icon(Icons.add, size: _mapToolLayerBtnIconSize),
            ),
          ),
          const SizedBox(height: _mapToolLayerBtnGap),
          SizedBox(
            width: _mapToolLayerBtnSize,
            height: _mapToolLayerBtnSize,
            child: MaterialButton(
              padding: EdgeInsets.zero,
              color: btnColor,
              onPressed: _moveCameraByZoomOut,
              child: const Icon(Icons.remove, size: _mapToolLayerBtnIconSize),
            ),
          ),
          const SizedBox(height: _mapToolLayerBtnSize),
          SizedBox(
            width: _mapToolLayerBtnSize,
            height: _mapToolLayerBtnSize,
            child: MaterialButton(
              padding: EdgeInsets.zero,
              color: btnColor,
              onPressed: _onLocationBtnPressed,
              child: const Icon(Icons.my_location, size: _mapToolLayerBtnIconSize),
            ),
          ),
          const SizedBox(height: _mapToolLayerBtnSize),
        ],
      ),
    );
  }

  String _formatDistance(double distance) {
    if (distance >= 1000) {
      return '${(distance / 1000).toStringAsFixed(2)}${AppLocalizations.sessionAmapDistanceKilometer}';
    } else {
      return '${distance.round()}${AppLocalizations.sessionAmapDistanceMeter}';
    }
  }

  void _getOuterMapList() async {
    final mapList = AppConfig.isMobileDevice ? await outermap.MapLauncher.installedMaps : <outermap.AvailableMap>[];
    if (mounted) {
      setState(() {
        _outerMapList.clear();
        _outerMapList.addAll(mapList);
      });
    }
  }

  String _getOuterMapName(outermap.AvailableMap availableMap) {
    switch (availableMap.mapType) {
      case outermap.MapType.amap:
        return AppLocalizations.sessionAmapNavigateNameAmap;
      case outermap.MapType.baidu:
        return AppLocalizations.sessionAmapNavigateNameBaidu;
      case outermap.MapType.tencent:
        return AppLocalizations.sessionAmapNavigateNameTencent;
      case outermap.MapType.apple:
        return AppLocalizations.sessionAmapNavigateNameApple;
      case outermap.MapType.google:
        return AppLocalizations.sessionAmapNavigateNameGoogle;
      default:
        return availableMap.mapName;
    }
  }

  void _moveCameraByZoomIn() {
    _mapMoveByBtnTap = true;
    _mapController?.moveCamera(CameraUpdate.zoomIn(), animated: true);
  }

  void _moveCameraByZoomOut() {
    _mapMoveByBtnTap = true;
    _mapController?.moveCamera(CameraUpdate.zoomOut(), animated: true);
  }

  bool _moveCameraToCenter() {
    final distance = AppOpener.countDistance(_pointCenter.latitude, _pointCenter.longitude, _mapCenterMarker.latitude, _mapCenterMarker.longitude);
    if (distance > 1) {
      AppConfig.logDebug(['中心位置坐标与中心标记坐标距离为$distance米，调用moveCamera()函数将会触发onCameraMoveEnd()事件函数，执行了moveCamera()操作']);
      _mapMoveByBtnTap = true;
      _mapController?.moveCamera(CameraUpdate.newCameraPosition(CameraPosition(target: LatLng(_pointCenter.latitude, _pointCenter.longitude), zoom: _mapZoomLevel)), animated: true);
      return true;
    } else {
      AppConfig.logDebug(['中心位置坐标与中心标记坐标距离为$distance米，调用moveCamera()函数不会触发onCameraMoveEnd()事件函数，不执行moveCamera()操做']);
      return false;
    }
  }

  void _searchPoiAndMoveTo({required double latitude, required double longitude}) {
    //搜索前先清空列表
    if (mounted) {
      setState(() {
        _pointList.clear();
        _pointIndex = 0;
      });
    }
    //开始搜索
    final originLocation = AppConfig.client.user.location;
    _appOpener.poiAmapSearch(latitude: latitude, longitude: longitude, origin: originLocation).then((value) {
      if (mounted) {
        setState(() {
          _pointList.clear();
          _pointList.addAll(value);
          _pointIndex = 0;
          //异步回调，中心标记_mapCenterMarker的坐标可能发生了变化，进行一次界面同步
          if (_pointList.isNotEmpty) {
            _pointCenter.updateByJson(_pointList.first.toJson());
            _moveCameraToCenter();
          }
        });
      }
      if (_scrollController.hasClients) {
        _scrollController.jumpTo(0);
      }
    });
  }

  void _onSendBtnPressed() async {
    //截图
    final value = await _mapController?.takeSnapshot(); //经测试，这是个同步操作，不需要添加loading
    if (value == null) {
      if (!mounted) return;
      Navigator.pop(context);
      return;
    }
    //编码与压缩
    SmartDialog.showLoading(msg: AppLocalizations.sessionAmapEncodeLoading);
    final bytes = await WidgetPicture.encodePictureBytes(WidgetPictureType.editAsJpg, value);
    SmartDialog.dismiss();
    if (bytes == null) {
      if (!mounted) return;
      Navigator.pop(context);
      return;
    }
    //上传
    SmartDialog.showLoading(msg: AppLocalizations.sessionAmapUploadLoading);
    final result = await AppConfig.client.attachUpload(type: Constant.metaTypeMessage, fileBytes: [bytes], mediaType: MediaType.parse(WidgetPicture.mediaTypeJpg));
    SmartDialog.dismiss();
    if (!mounted) return;
    if (result.ok) {
      _pointCenter.snapshotUrl = result.extra!.first.path;
      _pointCenter.zoomLevel = _mapZoomLevel;
      Navigator.pop(context, _pointCenter);
    } else {
      Navigator.pop(context);
    }
  }

  void _onNavigateItemTap(ThemeData themeData) {
    WidgetSheet.showBottomGridSheet<outermap.AvailableMap>(context, [_outerMapList], physics: AppConfig.globalScrollPhysics, cancelText: AppLocalizations.sessionAmapNavigateCancel, builder: (item) {
      return MaterialButton(
        padding: _navigatorItemInnerPadding,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Container(
              clipBehavior: Clip.hardEdge,
              decoration: const ShapeDecoration(shape: RoundedRectangleBorder(borderRadius: _navigatorItemIconRadius)),
              child: SvgPicture.asset(item.icon, width: _navigatorItemIconSize, height: _navigatorItemIconSize),
            ),
            const SizedBox(height: _navigatorItemInnerGap),
            Text(_getOuterMapName(item), style: themeData.textTheme.caption, overflow: TextOverflow.ellipsis),
          ],
        ),
        onPressed: () => Navigator.pop(context, item),
      );
    }).then((value) async {
      if (value == null) return;
      outermap.MapLauncher.showDirections(mapType: value.mapType, destination: outermap.Coords(_mapCenterMarker.latitude, _mapCenterMarker.longitude), destinationTitle: _pointCenter.description);
    });
  }

  void _onPoiListItemTap(Location target, int index) {
    if (mounted) {
      setState(() {
        _pointIndex = index;
        _pointCenter.updateByJson(target.toJson());
        //无需重新搜索，直接移动到中心位置
        _moveCameraToCenter();
      });
    }
  }

  void _onLocationBtnPressed() {
    if (widget.type == SessionAmapType.locationView) {
      //直接移动到中心标志
      _mapController?.moveCamera(CameraUpdate.newCameraPosition(CameraPosition(target: _mapCenterMarker, zoom: _mapZoomLevel)), animated: true);
    } else if (widget.type == SessionAmapType.locationShare) {
      final myLocation = AppConfig.client.user.location;
      //需要重新搜索，然后移动到中心位置
      _searchPoiAndMoveTo(latitude: myLocation.latitude, longitude: myLocation.longitude);
    }
  }

  void _onAmapLocationOk(Location location) {
    //同步当前位置到服务器
    final user = UserDirty();
    user.location = location;
    AppConfig.client.userUpdate(uid: AppConfig.client.user.id, fields: user.data);
  }

  void _onUserUpdateListener(EasyPacket packet) {
    //相当于点击了我的位置按钮
    _onLocationBtnPressed();
  }
}
