import 'package:flutter/material.dart';
import 'package:flutter_linkify/flutter_linkify.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'app_webview.dart';
import 'session_amap.dart';
import 'widget/widget_avator.dart';
import 'widget/widget_image.dart';

////
/// 消息渲染
///
class SessionMessage extends StatelessWidget {
  static const outerPaddingDefault = 8.0;

  final Message message;
  final Message? prevMsg;
  final bool showNick;
  final double nickSpace;
  final EdgeInsets outerBoxPadding;
  final EdgeInsets innerTxtPadding;
  final double innerTxtSpacing;
  final EdgeInsets innerBoxPadding;
  final Size bubbleTriangleSize;
  final BorderRadius bubbleRectangleRadius;
  final int Function(Message msg, List<String> urlList, List<String> heroList) onGetPictures;
  final String Function(Message msg) onGetImgHeroTag;

  const SessionMessage({
    Key? key,
    required this.message,
    required this.prevMsg,
    required this.showNick,
    this.nickSpace = 2,
    this.outerBoxPadding = const EdgeInsets.all(outerPaddingDefault),
    this.innerTxtPadding = const EdgeInsets.only(left: outerPaddingDefault + 2, right: outerPaddingDefault - 2, top: outerPaddingDefault, bottom: outerPaddingDefault),
    this.innerTxtSpacing = 0.15,
    this.innerBoxPadding = const EdgeInsets.all(outerPaddingDefault),
    this.bubbleTriangleSize = const Size(8, 8),
    this.bubbleRectangleRadius = const BorderRadius.all(Radius.circular(4)),
    required this.onGetPictures,
    required this.onGetImgHeroTag,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    // final themeData =Theme.of(context);
    final themeData = AppConfig.lightThemeData;
    final showTime = (prevMsg == null || (message.time - prevMsg!.time) > 3 * 60 * 1000);
    final nickStyle = Theme.of(context).textTheme.caption;
    Color bubbleColorL = themeData.brightness == Brightness.light ? Colors.white : Colors.white12;
    Color bubbleColorR = themeData.brightness == Brightness.light ? themeData.primaryColorLight : Colors.black38;
    Color? dividerColor = themeData.brightness == Brightness.light ? themeData.dividerTheme.color : const Color(0xFF565656);
    EdgeInsets bubblePadding = EdgeInsets.zero;
    switch (message.type) {
      case Constant.msgTypeText:
        bubblePadding = innerTxtPadding;
        break;
      case Constant.msgTypeImage:
      case Constant.msgTypeVoice:
      case Constant.msgTypeVideo:
        bubbleColorR = bubbleColorL;
        break;
      case Constant.msgTypeRealtimeVoice:
      case Constant.msgTypeRealtimeVideo:
      case Constant.msgTypeRealtimeShare:
      case Constant.msgTypeRealtimeLocal:
        bubblePadding = innerBoxPadding;
        break;
      case Constant.msgTypeShareHtmlPage:
      case Constant.msgTypeShareLocation:
      case Constant.msgTypeShareCardUser:
      case Constant.msgTypeShareCardTeam:
        bubbleColorR = bubbleColorL;
        bubblePadding = innerBoxPadding;
        break;
    }
    return Container(
      padding: outerBoxPadding,
      child: Column(
        children: [
          showTime ? Text(ComTools.formatDateTime(message.time, yyMMdd: true, hhmmss: true), style: nickStyle) : const SizedBox(),
          showTime ? SizedBox(height: outerBoxPadding.top + outerBoxPadding.bottom) : const SizedBox(),
          message.type == Constant.msgTypeSystem
              ? Padding(
                  padding: EdgeInsets.only(bottom: outerBoxPadding.bottom),
                  child: Text(message.body, style: nickStyle),
                )
              : (AppConfig.client.user.id == message.uid
                  ? Theme(
                      data: themeData,
                      child: _buildRightItem(context, themeData, bubbleColorR, bubblePadding, dividerColor, nickStyle),
                    )
                  : Theme(
                      data: themeData,
                      child: _buildLeftItem(context, themeData, bubbleColorL, bubblePadding, dividerColor, nickStyle),
                    )),
        ],
      ),
    );
  }

  Widget _buildLeftItem(BuildContext context, ThemeData themeData, Color bubbleColor, EdgeInsets bubblePadding, Color? dividerColor, TextStyle? nickStyle) {
    const avatorType = WidgetAvatorType.small;
    const avatorRadius2x = WidgetAvator.smallTypeRadius * 2;
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Row(
          children: [
            GestureDetector(
              behavior: HitTestBehavior.opaque,
              onTap: () => _onHeadTap(context),
              onLongPress: _onHeadLongPress,
              child: WidgetAvator(type: avatorType, url: message.displayIcon, urlList: message.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
            ),
            CustomPaint(size: bubbleTriangleSize, painter: _SessionsMessageBubblePainter(color: bubbleColor, right: false, down: showNick)),
          ],
        ),
        Expanded(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              showNick ? Text(message.displayNick, style: nickStyle, overflow: TextOverflow.ellipsis) : const SizedBox(),
              GestureDetector(
                behavior: HitTestBehavior.opaque,
                onTap: () => _onBodyTap(context),
                onLongPress: _onBodyLongPress,
                child: Container(
                  margin: showNick ? EdgeInsets.only(top: nickSpace) : EdgeInsets.zero,
                  padding: bubblePadding,
                  constraints: const BoxConstraints(minWidth: avatorRadius2x, minHeight: avatorRadius2x),
                  decoration: ShapeDecoration(color: bubbleColor, shape: RoundedRectangleBorder(borderRadius: bubbleRectangleRadius)),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: _buildItemBodyChildren(context, themeData, dividerColor),
                  ),
                ),
              ),
            ],
          ),
        ),
        SizedBox(width: avatorRadius2x + bubbleTriangleSize.width),
      ],
    );
  }

  Widget _buildRightItem(BuildContext context, ThemeData themeData, Color bubbleColor, EdgeInsets bubblePadding, Color? dividerColor, TextStyle? nickStyle) {
    const avatorType = WidgetAvatorType.small;
    const avatorRadius2x = WidgetAvator.smallTypeRadius * 2;
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        SizedBox(width: avatorRadius2x + bubbleTriangleSize.width),
        Expanded(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.end,
            children: [
              showNick ? Text(message.displayNick, style: nickStyle, overflow: TextOverflow.ellipsis) : const SizedBox(),
              GestureDetector(
                behavior: HitTestBehavior.opaque,
                onTap: () => _onBodyTap(context),
                onLongPress: _onBodyLongPress,
                child: Container(
                  margin: showNick ? EdgeInsets.only(top: nickSpace) : EdgeInsets.zero,
                  padding: bubblePadding,
                  constraints: const BoxConstraints(minWidth: avatorRadius2x, minHeight: avatorRadius2x),
                  decoration: ShapeDecoration(color: bubbleColor, shape: RoundedRectangleBorder(borderRadius: bubbleRectangleRadius)),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: _buildItemBodyChildren(context, themeData, dividerColor),
                  ),
                ),
              ),
            ],
          ),
        ),
        Row(
          children: [
            CustomPaint(size: bubbleTriangleSize, painter: _SessionsMessageBubblePainter(color: bubbleColor, right: true, down: showNick)),
            GestureDetector(
              behavior: HitTestBehavior.opaque,
              onTap: () => _onHeadTap(context),
              onLongPress: _onHeadLongPress,
              child: WidgetAvator(type: avatorType, url: message.displayIcon, urlList: message.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
            ),
          ],
        ),
      ],
    );
  }

  List<Widget> _buildItemBodyChildren(BuildContext context, ThemeData themeData, Color? dividerColor) {
    switch (message.type) {
      case Constant.msgTypeText:
        return [
          SelectableLinkify(
            text: message.body,
            style: themeData.textTheme.subtitle1?.copyWith(letterSpacing: innerTxtSpacing),
            options: const LinkifyOptions(humanize: false),
            onOpen: (link) => _onTxtLinkifyOpen(context, link),
          ),
        ];
      case Constant.msgTypeImage:
        return [
          Container(
            clipBehavior: Clip.hardEdge,
            constraints: const BoxConstraints(maxWidth: 100, maxHeight: 180),
            decoration: ShapeDecoration(shape: RoundedRectangleBorder(borderRadius: bubbleRectangleRadius)),
            child: Hero(
              tag: onGetImgHeroTag(message),
              child: WidgetImage(url: message.shareLinkUrl, onFormatUrl: AppConfig.formatAttachUrl),
            ),
          )
        ];
      case Constant.msgTypeVoice:
      case Constant.msgTypeVideo:
        return [];
      case Constant.msgTypeRealtimeVoice:
      case Constant.msgTypeRealtimeVideo:
        String mediaTimeText;
        Color? mediaItemColor;
        if (message.mediaGoing) {
          //正在通话中
          mediaTimeText = AppLocalizations.sessionMessageRealtimeGoing;
          mediaItemColor = const Color(0xFF1C881C);
        } else {
          final seconds = ((message.mediaTimeE - message.mediaTimeS) / 1000).round();
          if (message.mediaTimeS <= 0 || seconds < 1) {
            //未接通：开始时间<=0 或 总时长<1秒的为未接通的
            mediaTimeText = AppLocalizations.sessionMessageRealtimeMissed;
            // mediaItemColor = const Color(0xFF661C1C);
          } else {
            //已结束：格式化通话时长
            final h = (seconds / 3600).floor();
            final m = ((seconds % 3600) / 60).floor();
            final s = seconds - h * 3600 - m * 60;
            mediaTimeText = '${AppLocalizations.sessionMessageRealtimeTotal} ${h < 10 ? '0' : ''}$h:${m < 10 ? '0' : ''}$m:${s < 10 ? '0' : ''}$s';
            // mediaItemColor = const Color(0xFF1C1C44);
          }
        }
        return [
          Row(
            mainAxisSize: MainAxisSize.min,
            children: [
              Icon(message.type == Constant.msgTypeRealtimeVoice ? Icons.phone_in_talk_outlined : Icons.ondemand_video_outlined, color: mediaItemColor),
              const SizedBox(width: outerPaddingDefault),
              Text(mediaTimeText, style: themeData.textTheme.subtitle1?.copyWith(color: mediaItemColor)),
              const SizedBox(width: outerPaddingDefault / 2),
            ],
          ),
        ];
      case Constant.msgTypeRealtimeShare:
      case Constant.msgTypeRealtimeLocal:
        return [];
      case Constant.msgTypeShareHtmlPage:
        return [
          Align(alignment: Alignment.centerLeft, child: Text(message.title, style: themeData.textTheme.subtitle1, overflow: TextOverflow.ellipsis)),
          const SizedBox(height: outerPaddingDefault / 2),
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Expanded(child: Text(message.body, style: themeData.textTheme.caption, maxLines: 3, overflow: TextOverflow.ellipsis)),
              const SizedBox(width: outerPaddingDefault),
              WidgetAvator(type: WidgetAvatorType.large, url: message.shareIconUrl, urlList: message.shareHeadUrl, onFormatUrl: AppConfig.formatAttachUrl),
            ],
          ),
          const SizedBox(height: outerPaddingDefault),
          Divider(color: dividerColor),
          const SizedBox(height: outerPaddingDefault / 2),
          Align(alignment: Alignment.centerLeft, child: Text(AppLocalizations.modelConstMapText(message.type), style: themeData.textTheme.caption)),
        ];
      case Constant.msgTypeShareLocation:
        final location = message.shareLocation!;
        return [
          Align(alignment: Alignment.centerLeft, child: Text(location.description, style: themeData.textTheme.bodyText2, overflow: TextOverflow.ellipsis)),
          Align(alignment: Alignment.centerLeft, child: Text(location.address, style: themeData.textTheme.caption, overflow: TextOverflow.ellipsis)),
          const SizedBox(height: outerPaddingDefault),
          Row(
            children: [
              Expanded(
                child: SizedBox(
                  height: 120,
                  child: WidgetImage(url: location.snapshotUrl, onFormatUrl: AppConfig.formatAttachUrl),
                ),
              )
            ],
          ),
          const SizedBox(height: outerPaddingDefault / 2),
          Align(alignment: Alignment.centerLeft, child: Text(AppLocalizations.modelConstMapText(message.type), style: themeData.textTheme.caption)),
        ];
      case Constant.msgTypeShareCardUser:
      case Constant.msgTypeShareCardTeam:
        return [
          Row(
            children: [
              WidgetAvator(type: WidgetAvatorType.middle, single: message.type == Constant.msgTypeShareCardUser, url: message.shareIconUrl, urlList: message.shareHeadUrl, onFormatUrl: AppConfig.formatAttachUrl),
              const SizedBox(width: outerPaddingDefault),
              Expanded(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(message.title, style: themeData.textTheme.subtitle1, overflow: TextOverflow.ellipsis),
                    Text(message.body, style: themeData.textTheme.caption, overflow: TextOverflow.ellipsis),
                  ],
                ),
              ),
            ],
          ),
          const SizedBox(height: outerPaddingDefault),
          Divider(color: dividerColor),
          const SizedBox(height: outerPaddingDefault / 2),
          Align(alignment: Alignment.centerLeft, child: Text(AppLocalizations.modelConstMapText(message.type), style: themeData.textTheme.caption)),
        ];
    }
    return [];
  }

  void _onHeadTap(BuildContext context) {
    if (message.from == Constant.msgFromUser) {
      final ship = AppConfig.client.getUserShip(message.uid);
      if (ship is UserShip) {
        AppRouter.pushContactUserInfoPage(context, uid: ship.rid, fid: ship.fid, from: ship.from);
      }
    } else if (message.from == Constant.msgFromTeam) {
      AppRouter.pushContactUserInfoPage(context, uid: message.uid, fid: message.sid, from: Constant.shipFromMember);
    }
  }

  void _onHeadLongPress() {}

  void _onBodyTap(BuildContext context) {
    switch (message.type) {
      case Constant.msgTypeText:
        break;
      case Constant.msgTypeImage:
        final urlList = <String>[];
        final heroList = <String>[];
        final initIndex = onGetPictures(message, urlList, heroList);
        AppRouter.pushImageViewerPage(context, urlList: urlList, heroList: heroList, onFormatUrl: AppConfig.formatAttachUrl, initialPage: initIndex, scrollPhysics: AppConfig.globalScrollPhysics);
        break;
      case Constant.msgTypeVoice:
      case Constant.msgTypeVideo:
        break;
      case Constant.msgTypeRealtimeVoice:
      case Constant.msgTypeRealtimeVideo:
      case Constant.msgTypeRealtimeShare:
        if (message.mediaGoing) {
          if (message.mediaJoined.contains(AppConfig.client.user.id)) {
            AppRouter.pushSessionRealtimePage(context, message: message);
          } else {
            SmartDialog.showToast(AppLocalizations.sessionMessageRealtimeNotHint);
          }
        } else {
          SmartDialog.showToast(AppLocalizations.sessionMessageRealtimeEndHint);
        }
        break;
      case Constant.msgTypeRealtimeLocal:
        break;
      case Constant.msgTypeShareHtmlPage:
        AppRouter.pushAppWebViewPage(context, type: AppWebViewType.normal, title: message.title, url: message.shareLinkUrl);
        break;
      case Constant.msgTypeShareLocation:
        AppRouter.pushSessionAmapPage(context, type: SessionAmapType.locationView, location: message.shareLocation);
        break;
      case Constant.msgTypeShareCardUser:
        AppRouter.pushContactUserInfoPage(context, uid: message.shareCardId, fid: message.uid, from: Constant.shipFromShareCard);
        break;
      case Constant.msgTypeShareCardTeam:
        AppRouter.pushContactTeamInfoPage(context, tid: message.shareCardId, fid: message.uid, from: Constant.shipFromShareCard);
        break;
    }
  }

  void _onBodyLongPress() {}

  void _onTxtLinkifyOpen(BuildContext context, LinkableElement link) {
    AppRouter.pushSpecifiedUrlPage(context, url: link.url);
  }
}

class _SessionsMessageBubblePainter extends CustomPainter {
  final Color color;
  final bool right;
  final bool down;

  _SessionsMessageBubblePainter({required this.color, required this.right, required this.down});

  @override
  void paint(Canvas canvas, Size size) {
    //画笔
    final paint = Paint();
    paint.isAntiAlias = true; //抗锯齿
    paint.style = PaintingStyle.fill;
    paint.color = color;
    //三角形
    final path = Path();
    if (down) {
      if (right) {
        path.moveTo(0, size.height);
        path.lineTo(size.width, size.height * 0.5);
        path.lineTo(0, size.height * 2);
      } else {
        path.moveTo(0, size.height * 0.5);
        path.lineTo(size.width, size.height);
        path.lineTo(size.width, size.height * 2);
      }
    } else {
      if (right) {
        path.moveTo(0, 0);
        path.lineTo(size.width, size.height * 0.5);
        path.lineTo(0, size.height);
      } else {
        path.moveTo(0, size.height * 0.5);
        path.lineTo(size.width, 0);
        path.lineTo(size.width, size.height);
      }
    }
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) => false;
}
