import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'widget/widget_divider.dart';
import 'widget/widget_sheet.dart';

///
///我的信息编辑页类型
///
enum UsermeEditorType {
  ///账号
  no,

  ///密码
  pwd,

  ///昵称
  nick,

  ///签名
  desc,

  ///手机号
  phone,

  ///真实姓名
  name,

  ///身份证号
  card,

  ///性别
  sex,

  ///生日
  birth,

  ///地区信息
  addr,

  ///自定义字段
  customField,
}

///
/// 我的信息编辑页
///
class UsermeEditor extends StatefulWidget {
  ///页面标题
  final String fieldTitle;

  ///编辑器类型
  final UsermeEditorType editorType;

  ///输入框的标签
  final String? inputLabel;

  const UsermeEditor({Key? key, required this.fieldTitle, required this.editorType, this.inputLabel}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _UsermeEditorState();
}

class _UsermeEditorState extends State<UsermeEditor> {
  static const _sendCodeBtnFontSize = 17.0;
  //通用
  final _valueController = TextEditingController();
  //手机号验证码
  final _codeController = TextEditingController();
  final _newcodeController = TextEditingController();
  //地区相关
  final _countryController = TextEditingController();
  final _provinceController = TextEditingController();
  final _cityController = TextEditingController();
  final _districtController = TextEditingController();

  //其他
  bool _hasBodyPadding = true;
  bool _isValueEmpty = true;

  Timer? _sendCodeTimer;
  int _sendCodeWait = 0;
  bool _isCodeEmpty = true;

  Timer? _sendCodeTimerNew;
  int _sendCodeWaitNew = 0;
  bool _isCodeEmptyNew = true;

  bool _pwdObscure = true;
  int _sexValue = Constant.unknow;
  String _birthValue = '';

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['UsermeEditor.initState()', widget.editorType]);
    _initInputFieldValue();
  }

  @override
  void dispose() {
    AppConfig.logDebug(['UsermeEditor.dispose()', widget.editorType]);
    //通用
    _valueController.dispose();
    //手机号验证码
    _codeController.dispose();
    _newcodeController.dispose();
    //地区相关
    _countryController.dispose();
    _provinceController.dispose();
    _cityController.dispose();
    _districtController.dispose();
    //其他
    _sendCodeTimer?.cancel();
    _sendCodeTimerNew?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      appBar: _buildAppBar(themeData),
      body: _buildBody(themeData),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    return AppBar(
      title: Text('${AppLocalizations.usermeEditorTitlePreffix}${widget.fieldTitle.toLowerCase()}'),
      actions: [_buildSaveButton(themeData)],
    );
  }

  Widget _buildBody(ThemeData themeData) {
    return SafeArea(
      left: _hasBodyPadding,
      right: _hasBodyPadding,
      child: GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: ListView(
          physics: AppConfig.globalScrollPhysics,
          padding: _hasBodyPadding ? AppConfig.bodyGlobalPaddingMedium : EdgeInsets.zero,
          children: _buildBodyField(themeData),
        ),
        onTap: () {
          FocusScope.of(context).unfocus();
        },
      ),
    );
  }

  Widget _buildSaveButton(ThemeData themeData) {
    switch (widget.editorType) {
      case UsermeEditorType.no:
        if (AppConfig.client.user.no.isEmpty) {
          return TextButton(
            onPressed: _isValueEmpty ? null : _onSaveBtnPressed,
            child: Text(AppLocalizations.usermeEditorActionBtnSave),
          );
        } else {
          return const SizedBox();
        }
      case UsermeEditorType.pwd:
        if (AppConfig.client.user.phone.isNotEmpty) {
          return TextButton(
            onPressed: _isCodeEmpty || _isValueEmpty ? null : _onSaveBtnPressed,
            child: Text(AppLocalizations.usermeEditorActionBtnSave),
          );
        } else {
          return const SizedBox();
        }
      case UsermeEditorType.phone:
        if (AppConfig.client.user.phone.isNotEmpty) {
          return TextButton(
            onPressed: _isCodeEmpty || _isValueEmpty || _isCodeEmptyNew ? null : _onSaveBtnPressed,
            child: Text(AppLocalizations.usermeEditorActionBtnSave),
          );
        } else {
          return TextButton(
            onPressed: _isValueEmpty || _isCodeEmptyNew ? null : _onSaveBtnPressed,
            child: Text(AppLocalizations.usermeEditorActionBtnSave),
          );
        }
      case UsermeEditorType.sex:
        return TextButton(
          onPressed: _sexValue != Constant.userSexMale && _sexValue != Constant.userSexFemale ? null : _onSaveBtnPressed,
          child: Text(AppLocalizations.usermeEditorActionBtnSave),
        );
      default:
        return TextButton(
          onPressed: _onSaveBtnPressed,
          child: Text(AppLocalizations.usermeEditorActionBtnSave),
        );
    }
  }

  List<Widget> _buildBodyField(ThemeData themeData) {
    switch (widget.editorType) {
      case UsermeEditorType.no:
        if (AppConfig.client.user.no.isEmpty) {
          return [
            TextFormField(
              autofocus: true,
              controller: _valueController,
              keyboardType: TextInputType.emailAddress,
              maxLength: Validator.noLenMax,
              decoration: InputDecoration(
                labelText: widget.inputLabel ?? widget.fieldTitle,
                hintText: AppLocalizations.usermeEditorNoHint,
                helperText: AppLocalizations.usermeEditorNoHelper,
              ),
              onChanged: (value) {
                final empty = value.trim() == '';
                if (_isValueEmpty != empty) {
                  if (mounted) {
                    setState(() {
                      _isValueEmpty = empty;
                    });
                  }
                }
              },
            )
          ];
        } else {
          return [
            Text(AppLocalizations.usermeEditorNoHelper, style: themeData.textTheme.subtitle1, textAlign: TextAlign.center),
          ];
        }
      case UsermeEditorType.pwd:
        if (AppConfig.client.user.phone.isNotEmpty) {
          return [
            TextFormField(
              autofocus: true,
              controller: _codeController,
              keyboardType: TextInputType.phone,
              textInputAction: TextInputAction.next,
              maxLength: Validator.randcodeLen,
              decoration: InputDecoration(
                prefixIcon: const Icon(Icons.verified_user),
                labelText: AppLocalizations.usermeEditorPasswordCode,
                hintText: AppLocalizations.usermeEditorPasswordCodeHint,
                counterText: '',
                suffixIcon: TextButton(
                  onPressed: _sendCodeWait > 0 ? null : _onSendRandCodeBtnPressed,
                  child: Text(_sendCodeWait > 0 ? '$_sendCodeWait S' : AppLocalizations.usermeEditorRandcodeSend, style: const TextStyle(fontSize: _sendCodeBtnFontSize)),
                ),
              ),
              onChanged: (value) {
                final empty = value.trim() == '';
                if (_isCodeEmpty != empty) {
                  if (mounted) {
                    setState(() {
                      _isCodeEmpty = empty;
                    });
                  }
                }
              },
            ),
            AppConfig.cardGlobalSpaceMedium,
            TextFormField(
              obscureText: _pwdObscure,
              controller: _valueController,
              keyboardType: TextInputType.visiblePassword,
              maxLength: Validator.pwdLenMax,
              decoration: InputDecoration(
                prefixIcon: const Icon(Icons.lock),
                labelText: AppLocalizations.usermeEditorPasswordPwd,
                hintText: AppLocalizations.usermeEditorPasswordPwdHint,
                suffixIcon: IconButton(
                  icon: Icon(_pwdObscure ? Icons.visibility : Icons.visibility_off),
                  onPressed: () {
                    if (mounted) {
                      setState(() {
                        _pwdObscure = !_pwdObscure;
                      });
                    }
                  },
                ),
              ),
              onChanged: (value) {
                final empty = value.trim() == '';
                if (_isValueEmpty != empty) {
                  if (mounted) {
                    setState(() {
                      _isValueEmpty = empty;
                    });
                  }
                }
              },
            ),
          ];
        } else {
          return [
            Text(AppLocalizations.usermeEditorPasswordUnbind, style: themeData.textTheme.subtitle1, textAlign: TextAlign.center),
          ];
        }
      case UsermeEditorType.nick:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case UsermeEditorType.desc:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.mediumTextLenMax,
            minLines: 3,
            maxLines: 5,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case UsermeEditorType.phone:
        final result = <Widget>[];
        if (AppConfig.client.user.phone.isNotEmpty) {
          result.addAll([
            TextFormField(
              autofocus: true,
              controller: _codeController,
              keyboardType: TextInputType.phone,
              textInputAction: TextInputAction.next,
              maxLength: Validator.randcodeLen,
              decoration: InputDecoration(
                prefixIcon: const Icon(Icons.verified_user),
                labelText: AppLocalizations.usermeEditorPhoneOldCode,
                hintText: AppLocalizations.usermeEditorPhoneOldCodeHint,
                counterText: '',
                suffixIcon: TextButton(
                  onPressed: _sendCodeWait > 0 ? null : _onSendRandCodeBtnPressed,
                  child: Text(_sendCodeWait > 0 ? '$_sendCodeWait S' : AppLocalizations.usermeEditorRandcodeSend, style: const TextStyle(fontSize: _sendCodeBtnFontSize)),
                ),
              ),
              onChanged: (value) {
                final empty = value.trim() == '';
                if (_isCodeEmpty != empty) {
                  if (mounted) {
                    setState(() {
                      _isCodeEmpty = empty;
                    });
                  }
                }
              },
            ),
            AppConfig.cardGlobalSpaceMedium,
          ]);
        }
        result.addAll([
          TextFormField(
            autofocus: AppConfig.client.user.phone.isEmpty,
            controller: _valueController,
            keyboardType: TextInputType.phone,
            textInputAction: TextInputAction.next,
            maxLength: Validator.phoneLenMax,
            decoration: InputDecoration(
              prefixIcon: const Icon(Icons.phone_android),
              labelText: AppLocalizations.usermeEditorPhoneNewPhone,
              hintText: AppLocalizations.usermeEditorPhoneNewPhoneHint,
              counterText: '',
            ),
            onChanged: (value) {
              final empty = value.trim() == '';
              if (_isValueEmpty != empty) {
                if (mounted) {
                  setState(() {
                    _isValueEmpty = empty;
                  });
                }
              }
            },
          ),
          AppConfig.cardGlobalSpaceMedium,
          TextFormField(
            controller: _newcodeController,
            keyboardType: TextInputType.phone,
            textInputAction: TextInputAction.next,
            maxLength: Validator.randcodeLen,
            decoration: InputDecoration(
              prefixIcon: const Icon(Icons.verified_user_outlined),
              labelText: AppLocalizations.usermeEditorPhoneNewCode,
              hintText: AppLocalizations.usermeEditorPhoneNewCodeHint,
              counterText: '',
              suffixIcon: TextButton(
                onPressed: _isValueEmpty || _sendCodeWaitNew > 0 ? null : _onSendRandCodeBtnPressedNew,
                child: Text(_sendCodeWaitNew > 0 ? '$_sendCodeWaitNew S' : AppLocalizations.usermeEditorRandcodeSend, style: const TextStyle(fontSize: _sendCodeBtnFontSize)),
              ),
            ),
            onChanged: (value) {
              final empty = value.trim() == '';
              if (_isCodeEmptyNew != empty) {
                if (mounted) {
                  setState(() {
                    _isCodeEmptyNew = empty;
                  });
                }
              }
            },
          ),
        ]);
        return result;
      case UsermeEditorType.name:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case UsermeEditorType.card:
        return [
          TextFormField(
            autofocus: true,
            controller: _valueController,
            keyboardType: TextInputType.emailAddress,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: widget.inputLabel ?? widget.fieldTitle),
          )
        ];
      case UsermeEditorType.sex:
        return [
          Ink(
            color: themeData.cardColor,
            child: Column(
              children: [
                WidgetDivider.forFull,
                ListTile(
                  leading: Text(AppLocalizations.modelConstMapText(Constant.userSexMale), style: themeData.textTheme.subtitle1),
                  trailing: _sexValue == Constant.userSexMale ? Icon(Icons.check_outlined, color: themeData.colorScheme.secondary) : const SizedBox(),
                  onTap: () {
                    if (mounted) {
                      setState(() {
                        _sexValue = Constant.userSexMale;
                      });
                    }
                  },
                ),
                WidgetDivider.forFull,
                ListTile(
                  leading: Text(AppLocalizations.modelConstMapText(Constant.userSexFemale), style: themeData.textTheme.subtitle1),
                  trailing: _sexValue == Constant.userSexFemale ? Icon(Icons.check_outlined, color: themeData.colorScheme.secondary) : const SizedBox(),
                  onTap: () {
                    if (mounted) {
                      setState(() {
                        _sexValue = Constant.userSexFemale;
                      });
                    }
                  },
                ),
                WidgetDivider.forFull,
              ],
            ),
          ),
        ];
      case UsermeEditorType.birth:
        return [
          Ink(
            color: themeData.cardColor,
            child: Column(
              children: [
                WidgetDivider.forFull,
                ListTile(
                  leading: Text(AppLocalizations.usermeEditorChooseBirthday, style: themeData.textTheme.subtitle1),
                  title: Text(_birthValue, textAlign: TextAlign.right),
                  trailing: const Icon(Icons.expand_more_outlined),
                  onTap: () {
                    final now = DateTime.now();
                    int year = now.year;
                    int month = now.month;
                    int day = now.day;
                    try {
                      final ymdArr = _birthValue.split('-');
                      if (ymdArr.length == 3) {
                        year = int.parse(ymdArr[0]);
                        month = int.parse(ymdArr[1]);
                        day = int.parse(ymdArr[2]);
                      }
                    } catch (e) {
                      year = now.year;
                      month = now.month;
                      day = now.day;
                    }
                    final firstDate = DateTime(1949, 10, 1);
                    final initialDate = DateTime(year, month, day);
                    WidgetSheet.showBottomDateSheet(
                      context,
                      minimumDate: firstDate,
                      maximumDate: now,
                      initialDateTime: initialDate,
                      onDateTimeChanged: (value) {
                        if (mounted) {
                          setState(() {
                            _birthValue = ComTools.formatDateTime(value.millisecondsSinceEpoch, yyMMdd: true);
                          });
                        }
                      },
                    );
                  },
                ),
                WidgetDivider.forFull,
              ],
            ),
          ),
        ];
      case UsermeEditorType.addr:
        final isChinese = ['中华人民共和国', '中国', 'china', 'chinese'].any((element) => element == _countryController.text.trim().toLowerCase());
        return [
          TextFormField(
            controller: _countryController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: AppLocalizations.usermeEditorAddrCountry),
            onChanged: (value) {
              if (mounted) {
                setState(() {});
              }
            },
          ),
          AppConfig.cardGlobalSpaceSmall,
          TextFormField(
            autofocus: !isChinese,
            readOnly: isChinese,
            controller: _provinceController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: AppLocalizations.usermeEditorAddrProvince, suffixIcon: isChinese ? const Icon(Icons.expand_more_outlined) : null),
            onTap: isChinese ? _onShowCitySheetBtnPressed : null,
          ),
          AppConfig.cardGlobalSpaceSmall,
          TextFormField(
            readOnly: isChinese,
            controller: _cityController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: AppLocalizations.usermeEditorAddrCity, suffixIcon: isChinese ? const Icon(Icons.expand_more_outlined) : null),
            onTap: isChinese ? _onShowCitySheetBtnPressed : null,
          ),
          AppConfig.cardGlobalSpaceSmall,
          TextFormField(
            readOnly: isChinese,
            controller: _districtController,
            keyboardType: TextInputType.text,
            maxLength: Validator.shortTextLenMax,
            decoration: InputDecoration(labelText: AppLocalizations.usermeEditorAddrArea, suffixIcon: isChinese ? const Icon(Icons.expand_more_outlined) : null),
            onTap: isChinese ? _onShowCitySheetBtnPressed : null,
          )
        ];
      default:
        return [
          Text(widget.fieldTitle, style: themeData.textTheme.subtitle1, textAlign: TextAlign.center),
        ];
    }
  }

  void _initInputFieldValue() {
    switch (widget.editorType) {
      case UsermeEditorType.no:
        _valueController.text = AppConfig.client.user.no;
        break;
      case UsermeEditorType.nick:
        _valueController.text = AppConfig.client.user.nick;
        break;
      case UsermeEditorType.desc:
        _valueController.text = AppConfig.client.user.desc;
        break;
      case UsermeEditorType.name:
        _valueController.text = AppConfig.client.user.name;
        break;
      case UsermeEditorType.card:
        _valueController.text = AppConfig.client.user.card;
        break;
      case UsermeEditorType.sex:
        _hasBodyPadding = false;
        _sexValue = AppConfig.client.user.sex;
        break;
      case UsermeEditorType.birth:
        _hasBodyPadding = false;
        _birthValue = AppConfig.client.user.birth;
        break;
      case UsermeEditorType.addr:
        _countryController.text = AppConfig.client.user.country;
        _provinceController.text = AppConfig.client.user.province;
        _cityController.text = AppConfig.client.user.city;
        _districtController.text = AppConfig.client.user.district;
        break;
      default:
        break;
    }
  }

  void _onShowCitySheetBtnPressed() {
    FocusScope.of(context).unfocus();
    WidgetSheet.showBottomCitySheet(context, province: _provinceController.text, city: _cityController.text, district: _districtController.text).then((result) {
      if (result == null) return;
      if (mounted) {
        setState(() {
          _provinceController.text = result.province;
          _cityController.text = result.city ?? '';
          _districtController.text = result.county ?? '';
        });
      }
    });
  }

  void _onSaveBtnPressed() async {
    final user = UserDirty();
    String randcode = '';
    String newRandcode = '';
    switch (widget.editorType) {
      case UsermeEditorType.no:
        user.no = _valueController.text.trim();
        break;
      case UsermeEditorType.pwd:
        user.pwd = _valueController.text.trim();
        randcode = _codeController.text.trim();
        newRandcode = _newcodeController.text.trim();
        break;
      case UsermeEditorType.nick:
        user.nick = _valueController.text.trim();
        break;
      case UsermeEditorType.desc:
        user.desc = _valueController.text.trim();
        break;
      case UsermeEditorType.phone:
        user.phone = _valueController.text.trim();
        randcode = _codeController.text.trim();
        newRandcode = _newcodeController.text.trim();
        break;
      case UsermeEditorType.name:
        user.name = _valueController.text.trim();
        break;
      case UsermeEditorType.card:
        user.card = _valueController.text.trim();
        break;
      case UsermeEditorType.sex:
        user.sex = _sexValue;
        break;
      case UsermeEditorType.birth:
        user.birth = _birthValue;
        break;
      case UsermeEditorType.addr:
        user.country = _countryController.text.trim();
        user.province = _provinceController.text.trim();
        user.city = _cityController.text.trim();
        user.district = _districtController.text.trim();
        break;
      default:
        break;
    }
    SmartDialog.showLoading(msg: AppLocalizations.usermeEditorSaveLoading);
    final result = await AppConfig.client.userUpdate(uid: AppConfig.client.user.id, fields: user.data, code: randcode, newcode: newRandcode);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
    if (result.ok) {
      if (!mounted) return;
      Navigator.pop(context);
    }
  }

  void _onSendRandCodeBtnPressed() async {
    SmartDialog.showLoading(msg: AppLocalizations.usermeEditorSendLoading);
    final result = await AppConfig.client.sendRandcode(phone: AppConfig.client.user.phone);
    SmartDialog.dismiss();
    if (result.ok) {
      if (mounted) {
        setState(() {
          _sendCodeWait = 60; //开始倒计时
        });
      }
      _sendCodeTimer?.cancel();
      _sendCodeTimer = Timer.periodic(const Duration(seconds: 1), (timer) {
        if (_sendCodeWait > 0) {
          if (mounted) {
            setState(() {
              _sendCodeWait--;
            });
          }
        } else {
          _sendCodeTimer?.cancel();
          _sendCodeTimer = null;
        }
      });
    }
    SmartDialog.showToast(result.desc);
  }

  void _onSendRandCodeBtnPressedNew() async {
    SmartDialog.showLoading(msg: AppLocalizations.usermeEditorSendLoading);
    final result = await AppConfig.client.sendRandcode(phone: _valueController.text.trim());
    SmartDialog.dismiss();
    if (result.ok) {
      if (mounted) {
        setState(() {
          _sendCodeWaitNew = 60; //开始倒计时
        });
      }
      _sendCodeTimerNew?.cancel();
      _sendCodeTimerNew = Timer.periodic(const Duration(seconds: 1), (timer) {
        if (_sendCodeWaitNew > 0) {
          if (mounted) {
            setState(() {
              _sendCodeWaitNew--;
            });
          }
        } else {
          _sendCodeTimerNew?.cancel();
          _sendCodeTimerNew = null;
        }
      });
    }
    SmartDialog.showToast(result.desc);
  }
}
