import 'dart:io';

import 'package:camera/camera.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:image/image.dart' as imagetool;
import 'package:zxing_lib/common.dart' show HybridBinarizer;
import 'package:zxing_lib/datamatrix.dart' show DataMatrixReader;
import 'package:zxing_lib/maxicode.dart' show MaxiCodeReader;
import 'package:zxing_lib/aztec.dart' show AztecReader;
import 'package:zxing_lib/oned.dart' show CodaBarReader, Code128Reader, Code39Reader, Code93Reader, EAN13Reader, EAN8Reader, ITFReader, RSS14Reader, RSSExpandedReader, UPCAReader, UPCEReader;
import 'package:zxing_lib/pdf417.dart' show PDF417Reader;
import 'package:zxing_lib/qrcode.dart' show QRCodeReader;
import 'package:zxing_lib/zxing.dart' show BarcodeFormat, BinaryBitmap, RGBLuminanceSource, Reader, Result;

///
///二维码扫描
///
class WidgetQrscan extends StatefulWidget {
  final SystemUiOverlayStyle systemUiOverlayStyle;
  final List<BarcodeFormat> formats;
  final int scanMillis;
  final double cutRectSize;
  final double iconBtnSize;
  final bool Function(BuildContext context, String result) callback;
  const WidgetQrscan({
    Key? key,
    required this.systemUiOverlayStyle,
    this.formats = const [
      BarcodeFormat.QR_CODE,
      BarcodeFormat.AZTEC,
      BarcodeFormat.DATA_MATRIX,
      BarcodeFormat.PDF_417,
      BarcodeFormat.CODE_39,
      BarcodeFormat.CODE_93,
      BarcodeFormat.CODE_128,
      BarcodeFormat.EAN_8,
      BarcodeFormat.EAN_13,
    ],
    this.scanMillis = 500,
    this.cutRectSize = 280,
    this.iconBtnSize = 38,
    required this.callback,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _WidgetQrscanState();
}

class _WidgetQrscanState extends State<WidgetQrscan> with WidgetsBindingObserver {
  CameraController? _cameraController;
  FlashMode _flashMode = FlashMode.off;
  int _lastScanMills = 0;

  bool get _isMobileDevice => Platform.isAndroid || Platform.isIOS;

  @override
  void initState() {
    super.initState();
    _initController();
    WidgetsBinding.instance.addObserver(this);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    _cameraController?.dispose();
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    // print('=============> $state');
    if (_cameraController == null || !_cameraController!.value.isInitialized) return;
    if (state == AppLifecycleState.inactive) {
      _cameraController?.dispose();
    } else if (state == AppLifecycleState.resumed) {
      _cameraController?.dispose();
      _initController();
    }
  }

  @override
  Widget build(BuildContext context) {
    final queryData = MediaQuery.of(context);
    return Scaffold(
      extendBodyBehindAppBar: true,
      appBar: !_isMobileDevice
          ? AppBar()
          : AppBar(
              backgroundColor: Colors.transparent,
              systemOverlayStyle: widget.systemUiOverlayStyle,
              automaticallyImplyLeading: false,
              toolbarHeight: 0,
            ),
      body: Stack(
        alignment: Alignment.center,
        children: <Widget>[
          _cameraController == null
              ? Center(
                  child: SizedBox(
                    width: widget.iconBtnSize,
                    height: widget.iconBtnSize,
                    child: const CircularProgressIndicator(),
                  ),
                )
              : Positioned(
                  top: 0,
                  bottom: 0,
                  child: CameraPreview(_cameraController!),
                ),
          Positioned(
            bottom: (queryData.size.height - widget.cutRectSize) / 4,
            left: (queryData.size.width - widget.cutRectSize) / 2,
            child: IconButton(
              icon: Icon(Icons.highlight, color: Colors.white, size: widget.iconBtnSize),
              onPressed: _onToggleFlashMode,
            ),
          ),
          Positioned(
            bottom: (queryData.size.height - widget.cutRectSize) / 4,
            right: (queryData.size.width - widget.cutRectSize) / 2,
            child: IconButton(
              icon: Icon(Icons.close, color: Colors.white, size: widget.iconBtnSize),
              onPressed: () => Navigator.pop(context),
            ),
          ),
        ],
      ),
    );
  }

  void _initController() async {
    if (!_isMobileDevice) return;
    final cameras = await availableCameras();
    if (cameras.isNotEmpty) {
      _cameraController = CameraController(cameras.first, ResolutionPreset.high, enableAudio: false);
      await _cameraController?.initialize();
      if (mounted) {
        setState(() {
          _cameraController?.startImageStream(_onCameraStream);
        });
      }
    }
  }

  void _onCameraStream(CameraImage image) async {
    final current = DateTime.now().millisecondsSinceEpoch;
    if (current - _lastScanMills < widget.scanMillis) return;
    _lastScanMills = current; //更新时间
    final result = await compute(QRCodeDartScanMultiReader.decodeCameraImage, {'formats': widget.formats, 'image': image});
    if (result == null) return;
    if (!mounted) return;
    widget.callback(context, result.text);
  }

  void _onToggleFlashMode() {
    if (_flashMode == FlashMode.off) {
      _flashMode = FlashMode.torch;
    } else {
      _flashMode = FlashMode.off;
    }
    _cameraController?.setFlashMode(_flashMode);
  }
}

class QRCodeDartScanMultiReader {
  final List<BarcodeFormat> _formats;

  final List<Reader> _readers = [];

  QRCodeDartScanMultiReader(this._formats) {
    for (var format in _formats) {
      switch (format) {
        case BarcodeFormat.AZTEC:
          _readers.add(AztecReader());
          break;
        case BarcodeFormat.CODABAR:
          _readers.add(CodaBarReader());
          break;
        case BarcodeFormat.CODE_39:
          _readers.add(Code39Reader());
          break;
        case BarcodeFormat.CODE_93:
          _readers.add(Code93Reader());
          break;
        case BarcodeFormat.CODE_128:
          _readers.add(Code128Reader());
          break;
        case BarcodeFormat.DATA_MATRIX:
          _readers.add(DataMatrixReader());
          break;
        case BarcodeFormat.EAN_8:
          _readers.add(EAN8Reader());
          break;
        case BarcodeFormat.EAN_13:
          _readers.add(EAN13Reader());
          break;
        case BarcodeFormat.ITF:
          _readers.add(ITFReader());
          break;
        case BarcodeFormat.MAXICODE:
          _readers.add(MaxiCodeReader());
          break;
        case BarcodeFormat.PDF_417:
          _readers.add(PDF417Reader());
          break;
        case BarcodeFormat.QR_CODE:
          _readers.add(QRCodeReader());
          break;
        case BarcodeFormat.RSS_14:
          _readers.add(RSS14Reader());
          break;
        case BarcodeFormat.RSS_EXPANDED:
          _readers.add(RSSExpandedReader());
          break;
        case BarcodeFormat.UPC_A:
          _readers.add(UPCAReader());
          break;
        case BarcodeFormat.UPC_E:
          _readers.add(UPCEReader());
          break;
        case BarcodeFormat.UPC_EAN_EXTENSION:
          break;
      }
    }
  }

  static Result? decodeCameraImage(Map<String, dynamic> data) {
    final formats = data['formats'] as List<BarcodeFormat>;
    final image = data['image'] as CameraImage;
    imagetool.Image? img;
    switch (image.format.group) {
      case ImageFormatGroup.yuv420:
        img = _convertYUV420(image);
        break;
      case ImageFormatGroup.bgra8888:
        img = _convertBGRA8888(image);
        break;
      case ImageFormatGroup.jpeg:
        img = _convertJpeg(image);
        break;
      default:
        break;
    }
    if (img == null) return null;
    final multiReader = QRCodeDartScanMultiReader(formats);
    final source = RGBLuminanceSource(img.width, img.height, img.getBytes().buffer.asInt32List());
    final bitmap = BinaryBitmap(HybridBinarizer(source));
    for (final reader in multiReader._readers) {
      try {
        return reader.decode(bitmap);
      } catch (_) {}
    }
    return null;
  }

  /// CameraImage YUV420_888 -> PNG -> Image (compresion:0, filter: none)
  static imagetool.Image? _convertYUV420(CameraImage image) {
    if (image.planes.isEmpty) return null;
    final img = imagetool.Image(image.width, image.height); // Create Image buffer
    final plane = image.planes.first;
    const shift = 0xFF << 24;
    // Fill image buffer with plane[0] from YUV420_888
    for (var x = 0; x < image.width; x++) {
      for (var planeOffset = 0; planeOffset < image.height * image.width; planeOffset += image.width) {
        final pixelColor = plane.bytes[planeOffset + x];
        // color: 0x FF  FF  FF  FF
        //           A   B   G   R
        // Calculate pixel color
        final newVal = shift | (pixelColor << 16) | (pixelColor << 8) | pixelColor;
        img.data[planeOffset + x] = newVal;
      }
    }
    return img;
  }

  /// CameraImage BGRA8888 -> PNG
  static imagetool.Image? _convertBGRA8888(CameraImage image) {
    if (image.planes.isEmpty) return null;
    return imagetool.Image.fromBytes(
      image.width,
      image.height,
      image.planes.first.bytes,
      format: imagetool.Format.bgra,
    );
  }

  /// CameraImage JPEG -> JPG
  static imagetool.Image? _convertJpeg(CameraImage image) {
    if (image.planes.isEmpty) return null;
    return imagetool.decodeJpg(image.planes.first.bytes);
  }
}
